'''
Created on Oct 23, 2015
@author: hari
'''
from pycoils.lists.singly_linked_list import LinkedList
from pycoils.common import KeyValuePair
from pycoils.trees.bst import BinarySearchTree
from pycoils.trees.splaytree import SplayTree
import collections

class LinkedListHashBucket(collections.MutableMapping):
    '''
    A hash bucket is used to hold objects that 'hash to the same value' (collision) in a hash table. 
    This is hash a bucket using a LIST. This masquerades as a python dict in code where it is used.
    
    Note: HASHBUCKET ITERATION YIELDS KEYS. not the key value pairs in the bucket. 
    '''
    def __init__(self):
        self._list_of_kv_pairs = LinkedList() #we use our linked list!
    
    def __len__(self):
        '''
        The number of entries in the bucket!
        '''
        return self._list_of_kv_pairs.length
    
    def get(self, key, default = None):
        '''
        Get object associated with a key and on key miss return specified default. This is there in 
        Python dict and this class masquerades as dict, so we implement it.
        '''
        try:
            value = self[key]
            return value
        except KeyError:
            return default            
    
    def __getitem__(self, key):
        for kv_pair in self._list_of_kv_pairs:
            if kv_pair.key == key:
                return kv_pair.value
        raise KeyError('Key Error: %s ' % repr(key))
    
    def __delitem__(self, key):
        for kv_pair in self._list_of_kv_pairs:
            if kv_pair.key == key:
                self._list_of_kv_pairs.remove(kv_pair)
                return    
        raise KeyError('Key Error: %s ' % repr(key))
    
    def __setitem__(self, key, obj):
        for kv_pair in self._list_of_kv_pairs:
            if kv_pair.key == key:
                kv_pair.value = obj
                return    
        self._list_of_kv_pairs.append(KeyValuePair(key = key, value = obj))
  
    def __iter__(self):
        for kvpair in self._list_of_kv_pairs:
            yield kvpair.key

class BSTHashBucket(collections.MutableMapping):
    '''
    A hash bucket is used to hold objects that hash to the same value in a hash table. This is hash 
    bucket using a binary search tree. This masquerades as a python dict in code where it is used.
    Since bst is used as bucket datastructure, searches take O log(n) rather than O n
    
    Note: HASHBUCKET ITERATION YIELDS KEYS. not the key value pairs in the bucket. 
    '''
    def __init__(self):
        self._bst = BinarySearchTree()
    
    def __len__(self):
        '''
        The number of elements in the hash bucket
        '''
        return self._bst.node_count
    
    def get(self, key, default = None):
        '''
        Get object associated with a key and on key miss return specified default. This is 
        there in Python dict and this class masquerades as dict, we implement it.
        '''
        try:
            value = self[key]
            return value
        except KeyError:
            return default            
    
    def __getitem__(self, key):
        value = self._bst.find(key)
        if value == None:
            raise KeyError('Key Error: %s ' % repr(key))
        return value
        
    def __delitem__(self, key):
        if key in self._bst:
            self._bst.remove(key)
        else:
            raise KeyError('Key Error: %s ' % repr(key))
        
    def __setitem__(self, key, obj):
        if key in self._bst:
            self._bst.replace(key, obj)
        else:
            self._bst.insert(key, obj)
            
    def __iter__(self):
        for key, value in self._bst.inorder_traversal_with_stack():
            yield key

class SplayedHashBucket(collections.MutableMapping):
    '''
    A hash bucket is used to hold objects that hash to the same value in a hash table. 
    This is hash bucket using a splay tree. This masquerades as a python dict in code 
    where it is used. Since splat tree is used lookups on avg take Olog(n)
    
    Note: HASHBUCKET ITERATION YIELDS KEYS. not the key value pairs in the bucket. 
    '''
    def __init__(self):
        self._st = SplayTree()
    
    def __len__(self):
        return self._st.node_count
    
    def get(self, key, default = None):
        '''
        Get object associated with a key and on key miss return specified default. This is 
        there in Python dict and this class masquerades as dict, we implement it.
        '''
        try:
            value = self[key]
            return value
        except KeyError:
            return default            
    
    def __getitem__(self, key):
        value = self._st.find(key)
        if value == None:
            raise KeyError('Key Error: %s ' % repr(key))
        return value
        
    def __delitem__(self, key):
        if key in self._st:
            self._st.remove(key)
        else:
            raise KeyError('Key Error: %s ' % repr(key))
        
    def __setitem__(self, key, obj):
        if key in self._st:
            self._st.replace(key, obj)
        else:
            self._st.insert(key, obj)
            
    def __iter__(self):
        for key, value in self._st.inorder_traversal_with_stack():
            yield key


'''
Created on Aug 15, 2015
@author: hari
'''
from builtins import str
from builtins import object
from pycoils.common import KeyValuePair
from pycoils.stackNqueues.stack import Stack

class BSTTreeNode(object):
    '''
    A binary search tree node. For example, key can be a unique employee id and value is a python object 
    for employee.
    '''     
    __slots__ = 'kvpair', 'left_child', 'right_child', 'link_inversion_traversal_tag'
    def __init__(self, key = None, value = None, left_child = None, right_child = None):
        self.kvpair = KeyValuePair(key = key, value = value)
        self.right_child = right_child
        self.left_child = left_child
        self.link_inversion_traversal_tag = False #used for constant space traversal using link inversion.
    
    @property
    def key(self):
        return self.kvpair.key
    @key.setter
    def key(self, new_key):
        self.kvpair.key = new_key
    @property
    def value(self):
        return self.kvpair.value
    @value.setter
    def value(self, new_value):
        self.kvpair.value = new_value
    
    @property
    def has_left_child(self):
        if self.left_child != None:
            return True
        return False
    
    @property
    def has_right_child(self):
        if self.right_child != None:
            return True
        return False
    
    @property
    def is_leaf_node(self):
        return not self.has_left_child and not self.has_right_child
    
    @property
    def has_only_left_child(self):
        return not self.has_right_child and self.has_left_child
    
    @property
    def has_only_right_child(self):
        return not self.has_left_child and self.has_right_child
        
    def __str__(self):
        return '%s key[%s] value[%s]' % (self.__class__.__name__, str(self._kvpair.key), str(self._kvpair.value))

class BinarySearchTree(object):
    '''
    A binary search tree. At each node, the key > than keys in left sub-tree and <= keys in right 
    sub-tree.Duplicates are allowed as such. When deletion or find is done, the key encountered 
    first is returned and this depends on the tree structure and as such no guarantees are given 
    on this order.
    
    Note* Tree traversals yield(key, value) tuples 
          pre-order, in-order and post-order are implemented as constant-space link inversion traversals.               
          iter gives keys not values. This is so that the tree can be used in buckets for hash tables
    '''
    def __init__(self, root_node = None):
        self._root_node = root_node
        self._node_count = 0
        
        if self._root_node != None:
            self._node_count = 1

    @property    
    def node_count(self):
        return self._node_count
    
    def insert(self, key = None, obj = None):
        '''
        Add a key and its object value to the tree.
        '''
        if key == None or obj == None:
            return
        
        if self._root_node == None:
            self._root_node = BSTTreeNode(key = key, value = obj)
            self._node_count = 1
            return
        
        current_node = self._root_node
        parent_node = current_node
        
        while(current_node != None):
            parent_node = current_node
            if key >= current_node.key:
                current_node = current_node.right_child
            else:
                current_node = current_node.left_child
                
        if key >= parent_node.key:
            parent_node.right_child = BSTTreeNode(key= key, value = obj)
        else:
            parent_node.left_child = BSTTreeNode(key= key, value = obj)
        
        self._node_count = self._node_count + 1
        #    
            
    def _find_node_with_key(self, key):
        '''
        Returns the node with key and its parent node too as (node, parent). We use the parent often 
        in other methods so sending it together as a tuple saves a call to find the parent again.
        '''
        if self._root_node == None or key == None:
            return (None, None)
        current_node = self._root_node
        parent_node = None
        while (current_node != None):
            if current_node.key == key:
                return (current_node, parent_node)
            
            parent_node = current_node
            if key > current_node.key:
                current_node = current_node.right_child
            else:    
                current_node = current_node.left_child
        #while
        return (None, None)        
    
    
    def replace(self, key, obj):
        '''
        Replace the object associated with the key
        '''
        node_with_key, parent_node = self._find_node_with_key(key)
        
        if node_with_key  == None:
            raise KeyError('Key not found %s' % repr(key))
        
        node_with_key.value = obj
    
    def find(self, key):
        '''
        Find the value associated with key. Returns None if nothing is found.
        '''
        node_with_key, parent_node = self._find_node_with_key(key)
        
        if node_with_key  != None:
            return node_with_key.value
        else:
            return None
            
    def _find_parent_of_node(self, bst_tree_node):
        if bst_tree_node == None:
            return         
        search_key = bst_tree_node.key
        node, parent_of_node = self._find_node_with_key(search_key)
        return parent_of_node
    
    def _find_inorder_successor(self, node):
        if node == None:
            return None
        
        while node.left_child != None:
            node = node.left_child
        return node
        
    def remove(self, key):
        '''
        Remove the node with key.
        '''
        node_to_delete, parent_of_node_to_delete = self._find_node_with_key(key)
        
        if node_to_delete == None:
            return
    
        '''
        if the node to delete is a leaf node and it has a parent, set the parent child reference to 
        None. if node to delete does not have a parent, it is root, delete the node.
        update the count and return 
        '''
        if node_to_delete.is_leaf_node:
            if parent_of_node_to_delete != None:
                if parent_of_node_to_delete.left_child == node_to_delete:
                    parent_of_node_to_delete.left_child = None
                else:
                    parent_of_node_to_delete.right_child = None
            else:
                self._root_node = None
            
            self._node_count = self._node_count - 1
            return    
        
        if node_to_delete.has_only_left_child or node_to_delete.has_only_right_child:
            if parent_of_node_to_delete != None:
                if node_to_delete == parent_of_node_to_delete.left_child:
                    if node_to_delete.has_only_left_child :
                        parent_of_node_to_delete.left_child = node_to_delete.left_child
                    elif node_to_delete.has_only_right_child:
                        parent_of_node_to_delete.left_child = node_to_delete.right_child    
                elif node_to_delete == parent_of_node_to_delete.right_child:
                    if node_to_delete.has_only_left_child :
                        parent_of_node_to_delete.right_child = node_to_delete.left_child
                    elif node_to_delete.has_only_right_child:
                        parent_of_node_to_delete.right_child = node_to_delete.right_child
            else:#root with only one child
                if node_to_delete.has_only_left_child :
                    self._root_node = node_to_delete.left_child
                elif node_to_delete.has_only_right_child:
                    self._root_node = node_to_delete.right_child
            self._node_count = self._node_count - 1
            return
    
        #node has left and right children. find inorder successor of the right child of node to delete
        inorder_successor = self._find_inorder_successor(node_to_delete.right_child)
        parent_of_inorder_successor = self._find_parent_of_node(inorder_successor)
        
        right_child_of_inorder_successor = inorder_successor.right_child
        node_to_delete.kvpair = inorder_successor.kvpair
        
        if parent_of_inorder_successor != node_to_delete:
            parent_of_inorder_successor.left_child = right_child_of_inorder_successor
        else:
            parent_of_inorder_successor.right_child = right_child_of_inorder_successor
        self._node_count = self._node_count - 1
        return
    
    def traversal(self, want_pre_order = False, want_post_order = False, want_in_order = False):
        '''
        A yield is used to return the node elements while traversing the tree. Code is exact duplicate
        of link inversion traversal for tree in my Java library. It is available to read/download at
        http://harisankar-krishnaswamy.blogspot.in/2013/10/a-coder-post-link-inversion-traversal.html
        '''
        prev = None
        curr = self._root_node
        temp = None
        
        while 1==1:
            while curr != None: #fall down to left
                curr.link_inversion_traversal_tag = False
                if want_pre_order: 
                    yield (curr.key, curr.value)
                temp = curr.left_child
                curr.left_child = prev
                prev = curr
                curr = temp
                
            while prev != None and prev.link_inversion_traversal_tag == True: #Rise from right
                temp = prev.right_child #has our way up
                prev.right_child = curr #Reset pointer
                curr = prev         #Move up
                if want_post_order:
                    yield (curr.key, curr.value) 
                prev = temp
            
            if prev == None: 
                return
            else:
                    temp = prev.left_child
                    prev.left_child = curr
                    curr = prev;
                    prev = temp;
                    #We are comparing with pre-order traversal.
                    if want_in_order:
                        yield (curr.key, curr.value) 
                    curr.link_inversion_traversal_tag = True
                    temp = curr.right_child
                    curr.right_child = prev
                    prev = curr;
                    curr = temp;    
    #
    def has_key(self, key):
        node_with_key, parent = self._find_node_with_key(key)
        if node_with_key != None:
            return True
        return False
    
    def inorder_traversal_with_stack(self):
        '''
        this inorder traversal uses a stack and yields (key, value) pairs. this was added since link-
        inversion traversal modifies the tree during traversal. This is a bummer as we use tree as a
        hash bucket in hash tables. There are instances in MutableMapping where you iterate over the
        key (tree has changed temporarily) and do d[key] which duly fails. 
        '''
        stack = Stack()
        current_node = self._root_node
        while 1 == 1:
            while(current_node != None):
                stack.push(current_node)
                current_node = current_node.left_child
            if stack.size == 0:
                break
            popped_node = stack.pop()
            yield (popped_node.key, popped_node.value)
            current_node = popped_node.right_child
    
    def __iter__(self):#iter is used to loop through the keys. this is used in BSTHashBucket
        for node_key, node_value in self.inorder_traversal_with_stack():
            yield node_key
        


'''
Created on Aug 5, 2015
@author: topcat
'''
from __future__ import division
from builtins import str
from builtins import object
from past.utils import old_div
class Heap(object):
    """
    Implements heap class in python. Heap holds an array(Python list) so that for index i, 2*i is left child and 2*i+1 is right child 
    in heap tree (if Min heap i.e).
     0   1   2   3   4   5   6   7   8   9
    [#] [y] [c] [b] [e] [g] [f] [z] [a] [p] ........
    """
    def __init__(self, min_heap=True):
        """
        Initiate the heap's internal list storage. 
        if you want a max heap set minHeap = False
        """
        self._heapList = []
        self._heapList.append(None) # 0 index is not used.
        self._count = 0
        if min_heap is True:
            self._siftUp = self._sift_up_min_heap
            self._siftDown = self._sift_down_min_heap
        else:
            self._siftUp = self._sift_up_max_heap
            self._siftDown = self._sift_down_max_heap
        
        self._is_min_heap = min_heap
        
    @property
    def heap_size(self):
        return len(self._heapList) - 1#accomodate first unused entry TODO:TEST THIS 
    
    @property
    def is_min_heap(self):
        return self._is_min_heap
    
    def add_items(self, item_list):
        """
        Helper method that allows for multiple insertions.
        
        item_list: a Python list. 
        """
        if item_list is None:
            return
        for item in item_list:
            self.add_item(item)
        
    def add_item(self, item):
        """
        Add a Single item to the heap. Maintain heap condition.
        """
        if item is None: 
            return
        
        self._heapList.append(item)
        self._count = self._count + 1
        self._siftUp(self._count)
        
    
    def get(self):
        """
        Take an item from the top of the heap. Maintain heap condition.
        """
        if self._count == 0:
            return None
            
        item = self._heapList[1] #always we return index 1
        self._heapList[1] = self._heapList[self._count]
        del self._heapList[-1]
        self._siftDown()
        self._count = self._count - 1
        return item
    
    def _sift_up_min_heap(self, N):
        """
        MinHeap: Move the item at index N up until post condition is every heap item is lower than its children
        This operation maintains the min-heap condition after adding an item.
        """
        I = N
        parent = None
        while (I > 1):
            parent = self._heapList[old_div(I,2)]
            if self._heapList[I] < parent: # less than parent? make it the parent
                self._heapList[old_div(I,2)] = self._heapList[I]
                self._heapList[I] = parent
            else:
                # we reach a situation where the item is at I is not less than the parent. Break.
                break
            #Continue moving up the heap tree
            I = old_div(I, 2)
        #while
                
    def _sift_down_min_heap(self):
        """
        MinHeap: We move the item at index 1 down so that heap post condition every node in the heap is less than its own child nodes.
        Maintains heap property after removing an item from the heap.
        """
        I = 1
        C = 1
        while(2 * I < self._count):
            C = 2 * I
            if (C + 1)  < self._count: # has two children
                if self._heapList[C+1] < self._heapList[C]:
                    C = C + 1
            if self._heapList[I] > self._heapList[C]:
                temp = self._heapList[C]
                self._heapList[C] = self._heapList[I]
                self._heapList[I] = temp
            else:
                break
            I = C
    
    def _sift_up_max_heap(self, N):
        """
        MaxHeap: Move the item up until post condition is every item is less than its parent
        """
        I = N
        parent = None
        while (I > 1):
            parent = self._heapList[old_div(I,2)]
            if self._heapList[I] > parent: # less than parent? make it the parent
                self._heapList[old_div(I,2)] = self._heapList[I]
                self._heapList[I] = parent
            else:
                # we reach a situation where the item is not less than the parent. Break.
                break
            #Continue moving up the heap tree
            I = old_div(I, 2)
        #while
                
    def _sift_down_max_heap(self):
        """
        MaxHeap: We move the item at index 1 down so that post condition is every node item is greater than its children.
        """
        I = 1
        C = 1
        while(2 * I < self._count):
            C = 2 * I
            if (C + 1)  < self._count: # has two children
                if self._heapList[C+1] > self._heapList[C]:
                    C = C + 1
            if self._heapList[I] < self._heapList[C]:
                temp = self._heapList[C]
                self._heapList[C] = self._heapList[I]
                self._heapList[I] = temp
            else:
                break
            I = C
    
    def __str__(self):
        return str(self._heapList)
    #
    def __iter__(self):
        while self._count != 0:
            yield self.get()

    def _get_heap_as_list(self):
        """
        Returns the internal array representation of the heap.
        """
        return self._heapList


'''
Created on Aug 14, 2015
@author: topcat
'''
from __future__ import print_function
from builtins import range
import unittest
from pycoils.stackNqueues.queue import Queue

class Test_Len_Of_Empty_Queue(unittest.TestCase):
    
    def setUp(self):
        self._q = Queue()
    
    def test_len_of_empty_queue(self):
        self.assertEqual(len(self._q), 0, 'Empty Q length must be 0')
    
    def tearDown(self):
        del self._q

class Test_Front_Of_Empty_Queue(unittest.TestCase):
    
    def setUp(self):
        self._q = Queue()
    
    def test_front_of_empty_queue(self):
        self.assertEqual(self._q.front, None, 'Empty Q front must be None')
    
    def tearDown(self):
        del self._q

class Test_Dequeue_Of_Empty_Queue(unittest.TestCase):
    
    def setUp(self):
        self._q = Queue()
    
    def test_dequeue_of_empty_queue(self):
        self.assertEqual(self._q.dequeue(), None, 'Empty Q dequeue must be None')
        
    def tearDown(self):
        del self._q
    
class Test_Front_Len_Of_Predictable_Queue_With_1_Elements(unittest.TestCase):
    
    def setUp(self):
        self._q = Queue()
        self._q.enqueue(1)
    
    def test_front_of_queue_with_1_element(self):
        self.assertEqual(self._q.front, 1)
    
    def test_len_of_queue_with_1_element(self):
        self.assertEqual(len(self._q), 1)
    
    def tearDown(self):
        del self._q

class Test_Front_Len_While_Enqueuing_And_Dequeuing_A_Predictable_Queue_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._q = Queue()
        self.a_list = []  # use this to compare len size of queue as we enqueue and dequeue
        
    def test_front_len_with_enqueue_and_dequeue_of_4_element_queue(self):
         
        for i in range(4): # 0,1,2,3
            self.assertEqual(len(self._q), len(self.a_list))            
            self._q.enqueue(i)
            self.a_list.append(i)
            self.assertEqual(len(self._q), len(self.a_list))
            self.assertEqual(self._q.front, self.a_list[0]) #front remains same but we still check
        #DEQUEUE
        for i in range(4): # 0,1,2,3
            self.assertEqual(self._q.front, self.a_list[0])
            self.assertEqual(len(self._q), len(self.a_list))
            self.assertEqual(self._q.dequeue(), self.a_list[0])
            del self.a_list[0]
            self.assertEqual(len(self._q), len(self.a_list))
    
    def tearDown(self):
        del self._q
    
class Test_Iterator_Of_Predictable_Queue_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._q = Queue()
        self.a_list = [1,2,3,4]
        
        for element in self.a_list:
            self._q.enqueue(element)
    
    def test_iterator_of_queue_with_4_elements(self):
        for element in self._q:
            self.assertEqual(element, self.a_list[0], 'Iteration over Q did not give correct element')
            del self.a_list[0]
    def tearDown(self):
        del self._q
#Allows running as python run.
if __name__ == '__main__':
    print('queue tests')
    unittest.main()


'''
Created on Aug 5, 2015
@author: topcat
'''
from __future__ import print_function
from __future__ import division
from builtins import range
from past.utils import old_div
import unittest
from pycoils.trees.heap import Heap
from random import randint

def assert_max_heap_property(heap_list):
    """
    Tests a list representation of a heap for heap property.
    """ 
    count = len(heap_list) - 1
    index = 1
    limit = old_div(count, 2)

    while index <= limit:
        if heap_list[index] < heap_list[index *2]:
            return False
    
        if (index * 2 + 1) < count:
            if heap_list[2 * index + 1] > heap_list[index]:
                return False 
        
        index = index * 2
        
    return True


class MaxHeap_TestCase_Create_With_No_Element(unittest.TestCase):
    """
    Create only. No heap modification. No Elements.
    """    
    def setUp(self):
        self._heap = Heap(min_heap=False)
        
    def test_heap_status(self):
        self.assertFalse(self._heap.is_min_heap, 'heap must be a max heap.')
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        self.assertTrue([None] == self._heap._get_heap_as_list(), 'heap contents are not same as expected.')
    
    def tearDown(self):
        unittest.TestCase.tearDown(self) 

class MaxHeap_TestCase_Create_With_One_Element(unittest.TestCase):
    """
    Create only. No heap modification. Only one element.
    """    
    def setUp(self):
        self._heap = Heap(min_heap=False)
        self._heap.add_item(3)
        
    def test_heap_status(self):
        self.assertFalse(self._heap.is_min_heap, 'heap must be a max heap.')
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        self.assertTrue([None, 3] == self._heap._get_heap_as_list(), 'heap contents are not same as expected.')
    
    def tearDown(self):
        unittest.TestCase.tearDown(self) 

class MaxHeap_TestCase_Create_With_Predictable_SmallSet_Of_Elements(unittest.TestCase):
    """
    Create only. No heap modification. Small predictable set of elements.
    """
    def setUp(self):
        self._heap = Heap(min_heap=False)
        self._heap.add_items([3, 1, 2])
        
    def test_heap_status(self):
        self.assertFalse(self._heap.is_min_heap, 'heap must be a max heap.')
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        self.assertTrue([None, 3, 1, 2] == self._heap._get_heap_as_list(), 'heap contents are not same as expected.')
    
    def tearDown(self):
        unittest.TestCase.tearDown(self)

class MaxHeap_TestCase_1_Pop_With_Predictable_SmallSet_Of_Elements(unittest.TestCase):
    """
    Create only. No heap modification. Small predictable set of elements.
    """
    def setUp(self):
        self._heap = Heap(min_heap=False)
        self._heap.add_items([3, 1, 2])
        
    def test_heap_status(self):
        self.assertFalse(self._heap.is_min_heap, 'heap must be a max heap.')
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        self.assertTrue([None, 3, 1, 2] == self._heap._get_heap_as_list(), 'heap contents are not same as expected.')
        #Pop
        self._heap.get()
        #
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        self.assertTrue([None, 2, 1] == self._heap._get_heap_as_list(), 'heap contents are not same as expected.')
    
    def tearDown(self):
        unittest.TestCase.tearDown(self)

class MaxHeap_TestCase_2_Pops_With_Predictable_SmallSet_Of_Elements(unittest.TestCase):
    """
    Create only. No heap modification. Small predictable set of elements.
    """
    def setUp(self):
        self._heap = Heap(min_heap=False)
        self._heap.add_items([3, 1, 2])
        
    def test_heap_status(self):
        self.assertFalse(self._heap.is_min_heap, 'heap must be a max heap.')
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        self.assertTrue([None, 3, 1, 2] == self._heap._get_heap_as_list(), 'heap contents are not same as expected.')
        #Pop
        self._heap.get()
        self._heap.get()
        #
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        self.assertTrue([None, 1] == self._heap._get_heap_as_list(), 'heap contents are not same as expected.')
    
    def tearDown(self):
        unittest.TestCase.tearDown(self)
 

class MaxHeap_TestCase_1000_Pops_With_RandomSet_Of_1000_Elements(unittest.TestCase):
    """
    Create only. No heap modification. Small predictable set of elements.
    """
    def setUp(self):
        self._heap = Heap(min_heap = False)
    
    def test_heap_status_on_push_pop(self):
        self.assertFalse(self._heap.is_min_heap, 'heap must be a max heap.')
        
        for i in range(1000):
            self._heap.add_item(randint(0, 1000))
            self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        
        #pop all
        for i in range(1000):
            self._heap.get()
            self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        
    def tearDown(self):
        unittest.TestCase.tearDown(self)

class MaxHeap_TestCase_Iterator_Element(unittest.TestCase):
    """
    Create only. No heap modification. No Elements.
    """    
    def setUp(self):
        self._heap = Heap(min_heap = False)
        self._sortedListofElements = [40, 20, 10, 9, 4]
        self._heap.add_items([10, 9, 4, 20, 40])
    
    def test_heap_iterator(self):
        self.assertFalse(self._heap.is_min_heap, 'heap must be a max heap.')
        self.assertTrue(assert_max_heap_property(self._heap._get_heap_as_list()), 'heap property must be satisfied.')
        # Iterate through the heap and make sure the elements are present
        list_index = 0
        for element in self._heap:
            self.assertEqual(element, self._sortedListofElements[list_index], 'heap contents did not match up when iterating over a known list of elements.')
            list_index = list_index + 1
    
    def tearDown(self):
        unittest.TestCase.tearDown(self) 
                  
#Allows running as python run.
if __name__ == '__main__':
    print('MaxHeap tests')
    unittest.main()


'''
Created on Oct 25, 2015

@author: hari
'''
from __future__ import print_function
from builtins import str
from pycoils.hashtables.HashTables import SeperateChainHashTable
from pycoils.hashtables.hashbuckets import LinkedListHashBucket,\
    BSTHashBucket, SplayedHashBucket

def print_items_in_hash_table(hash_table):
    print('hash table size is %s' % str(len(hash_table)))
    for key in hash_table:
        print('key %s -> value %s' % (str(key), str(hash_table[key])))

if __name__ == '__main__':
    cht = SeperateChainHashTable()
    #
    # to control bucket data structure type use one of the following
    #
    #cht = SeperateChainHashTable(bucket_type_class = LinkedListHashBucket)
    #cht = SeperateChainHashTable(bucket_type_class = BSTHashBucket)
    #cht = SeperateChainHashTable(bucket_type_class = SplayedHashBucket)
    
    #add key, value pairs
    kvpairs = [(5, 5), (2, 2), (7, 7), (1, 1), (3, 3), (9, 9), (8, 8), (4, 4), (6, 6)]
    for kvpair in kvpairs:
        cht[kvpair[0]] = kvpair[1]
        
    print_items_in_hash_table(cht)
    
    #check if hash table has a key
    key_to_check = 10
    print('Key %s found in hashtable %s' % (str(key_to_check), key_to_check in cht))
    
    #get item with default if not found
    key_to_get = 20
    print('Get value for key %s value is %s' % (str(key_to_get), cht.get(key_to_get)))
    
    #replace
    key_to_replace_at = 1
    new_value = 111
    print('Replacing item at key %s with %s' % (str(key_to_replace_at), str(new_value)))
    cht[1] = 111
    
    #delete
    key_to_delete = 7
    print('Deleting key %s' % str(key_to_delete))
    del cht[7]
    print_items_in_hash_table(cht)


'''
Created on Oct 21, 2015
@author: hari
A hash table takes a key k (you decide for the object) and the object itself. We hash 
the key and get a bucket number B. This is just an index to a table. A bucket can implemented as a list 
or any other ds you like. A Bucket, which is masquerading as a python dict, holds multiple objects which 
happened to have keys k1 and k2 that mapped into the same bucket B. (collision)    
'''
from __future__ import division
from past.utils import old_div
from random import randrange
import collections
from pycoils.hashtables.hashbuckets import LinkedListHashBucket
            
class SeperateChainHashTable(collections.MutableMapping):
    '''
    The table uses ,as default, ListBasedHashBucket (Chain) for its buckets. This uses the MAD compression algorithm. 
    compressed_key = ((scale*key + shift) mod P) mod N. N is the size. P is a large prime > N. scale and shift are 
    random integers less than P, scale must be > 0. Mutable Mapping? have a look at "cat /usr/lib/python2.7/_abcoll.py" 
    Do not use this directly is advised in the file so we use from collections.MutableMapping as advised :-P Basically 
    we dont have to write the methods in Mutable mapping by inheriting.
    Note:-
    - Length of the hash table is the number of items in it.
    - You must specify bucket_type_class for controlling the datastructure used for separate chaining. Default is 
      LinkedListHashBucket. Other options are: SplayedHashBucket, BSTHashBucket
    '''
    def __init__(self, initial_capacity = 17, large_prime_P = 98764321261, load_factor_limit = 0.75, 
                                                                            bucket_type_class = LinkedListHashBucket):
        self._large_prime_P = large_prime_P
        self._table_max_size = initial_capacity
        self._scale = 1 + randrange(self._large_prime_P - 1)
        self._shift = randrange(self._large_prime_P)
        self._table = initial_capacity * [None]
        self._hash_table_items_count = 0
        self._load_factor_limit = load_factor_limit
        self._bucket_class = bucket_type_class
    
    @property
    def current_capacity(self):
        return len(self._table)
    
    def _find_bucket_for_key(self, key):#which bucket?
        hash_key = hash(key) #Pythons hash
        bucket_index = ((hash_key * self._scale + self._shift) % self._large_prime_P) % self._table_max_size 
        return bucket_index
    
    def _get_item_from_bucket(self, bucket_index, key):
        bucket = self._table[bucket_index]
        if bucket == None:
            raise KeyError('Key Error: %s' % repr(key))
        return bucket[key]
    
    def __getitem__(self, key):
        bucket_index = self._find_bucket_for_key(key)
        return self._get_item_from_bucket(bucket_index, key)
    
    def _delete_item_from_bucket(self, bucket_index, key):
        bucket = self._table[bucket_index]
        if bucket == None:
            raise KeyError('Key Error: %s' % repr(key))
        del bucket[key]
        self._hash_table_items_count = self._hash_table_items_count - 1
        
    def __delitem__(self, key):
        bucket_index = self._find_bucket_for_key(key)
        return self._delete_item_from_bucket(bucket_index, key)
    
    def _set_item_in_bucket(self, bucket_index, key, obj):
        bucket = self._table[bucket_index]
        if bucket == None:
            bucket = self._bucket_class()
            self._table[bucket_index] = bucket
        items_count_in_bucket = len(bucket)
        bucket[key] = obj 
        updated_items_count_in_bucket = len(bucket)
        if items_count_in_bucket < updated_items_count_in_bucket:#bucket did not have the item. was not a replace
            self._hash_table_items_count = self._hash_table_items_count + 1
    
    def __setitem__(self, key, obj):
        bucket_index = self._find_bucket_for_key(key)
        self._set_item_in_bucket(bucket_index, key, obj)
        if old_div(self._hash_table_items_count, float(len(self._table))) >= self._load_factor_limit:
            self._resize(self._table_max_size * 2)
             
    def __iter__(self):
        for bucket in self._table:
            if bucket != None:
                for key in bucket:
                    yield key
                    
    def __len__(self):
        return self._hash_table_items_count
    
    def has_key(self, key):
        try:
            self[key]
        except KeyError:
            return False
        return True
    
    def _resize(self, new_capacity):
        list_of_key_value_pairs = list(self.items())
        self._table_max_size = new_capacity
        self._table = new_capacity * [None]
        self._hash_table_items_count = 0
        
        for key, obj in list_of_key_value_pairs:
            self[key] = obj 


'''
Created on Oct 29, 2015
@author: hari
'''
from __future__ import print_function
from builtins import range
import unittest
from pycoils.stackNqueues.priorityqueue import PriorityQueue
import string

class Priority_Q_Min_Heap_Test_Empty_Q_Length(unittest.TestCase):
    def setUp(self):
        self._priorityQ = PriorityQueue(reverse_priority = True)
    
    def test_priority_q_length(self):
        self.assertEqual(len(self._priorityQ), 0, 'Empty priority Q length must be 0')
    
    def tearDown(self):
        self._priorityQ = None

class Priority_Q_Min_Heap_Test_Empty_Q_Iteration(unittest.TestCase):
    def setUp(self):
        self._priorityQ = PriorityQueue(reverse_priority = True)
    
    def test_priority_q_iteration(self):
        list_of_q_entries = []
        for item, priority in self._priorityQ:
            list_of_q_entries.append((item, priority))
            
        self.assertEqual(len(list_of_q_entries), 0, 'Empty priority Q iteration cannot yield anything')

    def tearDown(self):
        self._priorityQ = None

class Priority_Q_Min_Heap_Test_Q_Iteration(unittest.TestCase):
    def setUp(self):
        self._priorityQ = PriorityQueue(reverse_priority = True)
    
    def test_priority_q_iteration(self):
        list_of_q_entries = []
        for item, priority in self._priorityQ:
            list_of_q_entries.append((item, priority))
            
        self.assertEqual(len(list_of_q_entries), 0, 'Empty priority Q iteration cannot yield anything')

    def tearDown(self):
        self._priorityQ = None

class Priority_Q_Min_Heap_Test_Get_With_Empty_Q(unittest.TestCase):
    def setUp(self):
        self._priorityQ = PriorityQueue(reverse_priority = True)
        
    def test_priority_q_test_add_element(self):
        self.assertEqual(self._priorityQ.get(), None, 'Empty Q must return None on get op')
    
    def tearDown(self):
        self._priorityQ = None

class Priority_Q_Min_Heap_Test_Q_Add(unittest.TestCase):
    def setUp(self):
        self._item = 'Some Item'
        self._priority = 2
        self._priorityQ = PriorityQueue(reverse_priority = True)
        
    def test_priority_q_test_add_element(self):
        self._priorityQ.add_item(self._item, self._priority)
        self.assertEqual(len(self._priorityQ), 1, 'Priority Q length must be 1')
    
    def tearDown(self):
        self._priorityQ = None

class Priority_Q_Min_Heap_Test_Q_Add_And_Get_With_Single_Entry(unittest.TestCase):
    def setUp(self):
        self._item = 'Some Item'
        self._priority = 2
        self._priorityQ = PriorityQueue(reverse_priority = True)
        
    def test_priority_q_test_add__and_get(self):
        self._priorityQ.add_item(self._item, self._priority)
        item = self._priorityQ.get()
        self.assertEqual(item, self._item, 'Get op on priority Q did not yield expected item')
    
    def tearDown(self):
        self._priorityQ = None

class Priority_Q_Min_Heap_Test_Q_Iter_With_Single_Entry(unittest.TestCase):
    def setUp(self):
        self._item = 'Some Item'
        self._priority = 2
        self._priorityQ = PriorityQueue(reverse_priority = True)
        
    def test_priority_q_test_iter(self):
        self._priorityQ.add_item(item = self._item, priority = self._priority)
        expected_list_of_entries = [(self._item, self._priority)]
        list_of_q_entries = []
        
        for item, priority in self._priorityQ:
            list_of_q_entries.append((item, priority))
        
        self.assertEqual(list_of_q_entries, expected_list_of_entries, 'Priority Q iteration did not yield expected entries')
        
    def tearDown(self):
        self._priorityQ = None

class Priority_Q_Min_Heap_Test_Add_Get_Multiple_Entries(unittest.TestCase):
    def setUp(self):
        self._item_priority_entries = []
        
        for letter in string.uppercase:#build letter, ord(letter) tuples
            self._item_priority_entries.append((letter, ord(letter)))
             
        self._priorityQ = PriorityQueue(reverse_priority = True)
        
        self._expected_order_of_items_from_get = []
        for letter in string.uppercase:#build Z, Y, X, .....C, B , A because this is a max heap.
            self._expected_order_of_items_from_get.append(letter)
        
    def test_add_get_multiple_entries(self):
        for item, priority in self._item_priority_entries:
            self._priorityQ.add_item(item, priority)
            
        items_from_q = []
        for i in range(len(self._item_priority_entries)):#get this many items from the Q
            items_from_q.append(self._priorityQ.get())
            
        self.assertEqual(items_from_q, self._expected_order_of_items_from_get, 'Items from priority Q did not match expected list.')
    def tearDown(self):
        self._priorityQ = None


class Priority_Q_Min_Heap_Test_Iteration_With_Multiple_Entries(unittest.TestCase):
    def setUp(self):
        self._item_priority_entries = []
        
        for letter in string.uppercase:#build letter, ord(letter) tuples
            self._item_priority_entries.append((letter, ord(letter)))
             
        self._priorityQ = PriorityQueue(reverse_priority = True)
        
        self._expected_item_priority_pairs = []
        for letter in string.uppercase:
            self._expected_item_priority_pairs.append((letter, ord(letter)) )
        
    def test_iteration_with_multiple_entries(self):
        for item, priority in self._item_priority_entries:
            self._priorityQ.add_item(item, priority)
            
        items_and_priorities_pairs_from_q = []
        for item, priority in self._priorityQ:
            items_and_priorities_pairs_from_q.append((item, priority))
        
        self.assertEqual(items_and_priorities_pairs_from_q, self._expected_item_priority_pairs, 'Iteration on priority Q did not yield expected (item, priority) list')
        
    def tearDown(self):
        self._priorityQ = None

if __name__ == '__main__':
    print('Priority Q with Min Heap tests')
    unittest.main()


'''
Created on Oct 28, 2015
@author: hari
'''
from builtins import object
from pycoils.hashtables.HashTables import SeperateChainHashTable
from pycoils.hashtables.hashbuckets import SplayedHashBucket

class UpTreeNode(object):
    '''
    Uptrees: Every node in the tree holds a reference to a parent node. Every node also holds the 
    number of nodes in the tree rooted at that node. This is used in disjoint set union find operations.
    '''
    __slots__ = 'node_element', 'node_count', 'parent_node' #attribute properties are already added by python slots
    def __init__(self, node_element = None, node_count = 1, parent_node = None):
        self.node_element = node_element
        self.node_count = node_count
        self.parent_node = parent_node



'''
Created on Oct 28, 2015
@author: hari
'''
from builtins import object
from pycoils.hashtables.hashbuckets import SplayedHashBucket
from pycoils.hashtables.HashTables import SeperateChainHashTable
from pycoils.trees.uptree import UpTreeNode

class DisjointSetWithUnion(object):
    '''
    A disjoint set holds a set of items. The set may also hold subset of items.
    
    A disjoint set DJS has a number of elements from a fixed Universe U. DJS knows all the elements in it.
    The elements in DJS may themselves form subsets. i.e U = { a, b, c, d, r, f, g}. DJS can be
    { {a, g}, {b}, {d}, {c, r, f} }. Disjoint set allows 3 operations.
    
    1) make_set(x) returns a new set {x}
    2) find(x) return R such that x is in set R. for example {c, r, f} be identified by c. then find(r) = c
       The set to which an item belongs is represented by one of the items in the set. This representative item is usually at
       the root node of the uptree that links together all the items in the set.
    3) union(s, t): merges the smaller set into the larger. {c} U {r, f} = {c, r, f}
    for more info please refer Datastructures and their algorithms by Harry Lewis and Larry Denenberg
    
    Uptree nodes are used to hold the individual items of the set.
    
    1) Each item needs to be known in the set. We use a hashtable where table[item] = UpTree Node for the item
    2) make_set(item) just creates an uptree node with item. Each node starts off as a tree rooted at node.
    3) Iter on Disjoint set results in yielding (item, item's set) tuples.    
    '''
    def __init__(self):
        self._table_of_uptrees = SeperateChainHashTable(bucket_type_class = SplayedHashBucket)
        
    @property
    def size(self):
        return len(self._table_of_uptrees)
        
    def make_set(self, item = None):
        if item == None:
            return None
        uptree_node = UpTreeNode(node_element = item)#single node tree
        self._table_of_uptrees[item] = uptree_node
        return uptree_node
    
    def _uptree_find_with_path_compression(self, item = None):
        '''
        Return the root of the uptree to which item belongs. Compress the path before leaving.
        '''
        uptree_node_with_item = self._table_of_uptrees.get(item, None)
        if uptree_node_with_item == None:
            raise KeyError('Key Error: %s ' % repr(item))
        #before returning the root node element, we set the parent of the original node to the parent.
        current_node = uptree_node_with_item
        while current_node.parent_node != None:
            current_node = current_node.parent_node
        #we have root node
        root_of_uptree = current_node
        #compress path
        current_node = uptree_node_with_item
        parent_of_current_node = current_node.parent_node 
        while current_node != root_of_uptree:
            parent_of_current_node = current_node.parent_node
            current_node.parent_node = root_of_uptree
            current_node = parent_of_current_node
            
        return root_of_uptree
        
    def find(self, item):
        '''
        returns the set to which this item belongs. The set to which an item belongs is represented by the item at root of the set.
        '''
        try:
            root_node = self._uptree_find_with_path_compression(item)
        except KeyError:
            return  None
        return root_node.node_element
        
    def union(self, item_1 = None, item_2 = None):
        '''
        Finds the set for item_1 and item_2. Merges the two and returns the larger set.
        '''
        if item_1 == None or item_2 == None:
            return None
        
        uptree_1 = self._uptree_find_with_path_compression(item = item_1)#root actually
        uptree_2 = self._uptree_find_with_path_compression(item = item_2)#root actually
        
        #return the remaining dominating tree as first element of the tuple
        if uptree_1.node_count >= uptree_2.node_count:
            uptree_2.parent_node = uptree_1
            uptree_1.node_count = uptree_1.node_count + uptree_2.node_count
            return uptree_1.node_element
        else:
            uptree_1.parent_node = uptree_2
            uptree_2.node_count = uptree_2.node_count + uptree_1.node_count
            return uptree_2
    
    def __iter__(self):
        '''
        Iter on Disjoint set results in yielding (item, item's set) tuples
        '''
        item_in_table = None
        for key in self._table_of_uptrees:
            item_in_table = self._table_of_uptrees[key].node_element#hash table entries are uptree nodes
            yield (item_in_table, self.find(item_in_table))


'''
Created on Oct 28, 2015
@author: hari
'''
from __future__ import print_function
import unittest
from pycoils.trees.uptree import UpTreeNode

class UpTreeNode_Test_Default_Attributes(unittest.TestCase):
    def setUp(self):
        self._uptree_node = UpTreeNode()
    
    def test_default_attributes(self):
        self.assertEqual(self._uptree_node.node_count, 1, 'UpTree node default node count was not correct')
        self.assertEqual(self._uptree_node.parent_node, None, 'UpTree node default parent must be None')
        self.assertEqual(self._uptree_node.node_element, None, 'UpTree node default element must be None')
    def tearDown(self):
        self._uptree_node = None
    
class UpTreeNode_Test_Attributes_With_Predefined_Values(unittest.TestCase):
    def setUp(self):
        self._element = 2
        self._parent_node = UpTreeNode(node_element = 1)
        self._uptree_node = UpTreeNode(node_element = self._element, node_count = 1, 
                                                                     parent_node = self._parent_node)
    def test_default_attributes(self):
        self.assertEqual(self._uptree_node.node_count, 1, 'UpTree node node count was not correct')
        self.assertEqual(self._uptree_node.parent_node, self._parent_node, 'UpTree node parent did not add up')
        self.assertEqual(self._uptree_node.node_element, self._element, 'UpTree node element does not matchup')
        
    def tearDown(self):
        self._uptree_node = None

if __name__ == '__main__':
    print('UpTreeNode tests')
    unittest.main()


'''
Created on Nov 8, 2015
@author: hari
'''
from __future__ import print_function
from builtins import object
from functools import total_ordering
from pycoils.misc.internstore import InternStore

@total_ordering
class Country(object):
    def __init__(self, country_name = 'US'):
        self._country_name = country_name
    
    @property
    def country_name(self):
        return self._country_name
        
    def __str__(self):
        return '%s' % self._country_name
    
    def __eq__(self, other):
        if other == None or type(other) != type(self):
            return False
        return self._country_name == other.country_name
    
    def __lt__(self, other):
        if other == None or type(other) != type(self):
            return False
        return self._name < other.country_name
    
    def __hash__(self):
        return hash(self._country_name)
    
if __name__ == '__main__':
    US = Country(country_name = 'US')
    IN = Country(country_name = 'IN')
    
    intern_store = InternStore()
    intern_store.intern(obj = US)
    intern_store.intern(obj = IN)
    
    print('Id of original US object is %d' % id(US))
    
    duplicate_country_US = Country(country_name = 'US')
    print('id of duplicate US object is %d' % id(duplicate_country_US))
    print('Interning this new object')
    duplicate_country_US = intern_store.intern(obj = duplicate_country_US)
    print('After interning we get the original object with id %d' % id(duplicate_country_US))
     
    print('We might as well do this country = intern_store.intern(Country(name=\'US\'))')


'''
Created on Aug 12, 2015
@author: hari
'''
from builtins import str
from builtins import object
import collections
class LinkedNode(object):
    """
    A node in a Linked list.
    """
    __slots__ = 'element', 'next_node'
    def __init__(self, element = None, next_node = None):
        '''
        a node is created with the element object and the next node. If any of these
        are not available, use the default value None.
        '''
        self.next_node = next_node
        self.element = element
        
    def __str__(self):
        return '%s %s' % (self.__class__.__name__, self._element)
    
    def __repr__(self):
        return '%s(element = %s)' % (self.__class__.__name__, self._element)
        
class LinkedList(collections.Sequence):
    '''
    A Linked list implementation. More pythonic by using iterator features.
    Note:
    1) Iterating over the list does not take out the nodes.
    2) Node with duplicate elements are allowed.
    3) LinkedNode A == LinkedNode B does not compare their elements. You must do 
       A.element == B.element How element objects match is up to the application. 
    '''
    def __init__(self):
        self._head = None
        self._tail = self._head
        self._length = 0
            
    @property
    def length(self):
        return self._length
    
    @property
    def head(self):
        if self._head == None:
            return None
        return self._head.element
    
    @property
    def tail(self):
        if self._tail == None:
            return None
        return self._tail.element
        
    def append(self, element):
        """
        Adds an element to the end of the list as a Node.
        """
        if self._head is None:
            self._head = LinkedNode(element)
            self._tail = self._head
        else:
            new_node = LinkedNode(element)
            self._tail.next_node = new_node
            self._tail = new_node
        
        self._length = self._length + 1
    #
    
    def index(self, element):
        '''
        Returns -1 if element is not present in list. If present function returns the
        0 based index of the element in the list.
        '''
        if self._length == 0:
            return -1
         
        index = 0
        current_node = self._head
        while (current_node is not None):
            if current_node.element == element:
                return index
            current_node = current_node.next_node
            index = index + 1
        return -1
    #
    def insert_at(self, index, element):
        if index <= 0 or index >= self._length: #insert at head or at tail
            self.append(element) #takes care of length, head, tail etc
            return
        #index is somewhere in between head and tail locations. we find the node at index - 1
        count = 0
        till_where = index - 1 #calculate this before hand rather than leave it to the while condition. Helps in very large index.
        current_node = self._head
        
        while count != till_where:
            current_node = current_node.next_node
            count = count + 1
        temp = current_node.next_node
        current_node.next_node = LinkedNode(element = element, next_node = temp)
        self._length = self._length + 1
            
    def extend(self, linked_list):
        '''
        extend takes another LinkedList instance and adds each element of linked_list as a NEW
        node of this list.
        '''
        if linked_list is None:
            return   
        for element in linked_list:
            self.append(element)
    #
    
    def remove(self, element):
        if self._length == 0:
            return
        
        if self._head.element == element:
            if self._head.next_node == None:
                self._head = None
                self._tail = None
            else:
                self._head = self._head.next_node
            self._length = self._length - 1
            return
        
        current_node = self._head
        prev_node = current_node
        
        while (current_node != None):
            if current_node.element != element:
                prev_node = current_node
                current_node = current_node.next_node
            else:
                break
        
        if current_node == None:#did not find the element in the list.
            return
        #element found. it is in current_node.
        prev_node.next_node = current_node.next_node
        if prev_node.next_node == None: #Prev is the tail now.
            self._tail = prev_node
        self._length = self._length - 1
    #
    def __getitem__(self, index):
        if self._length == 0:#empty list
            raise IndexError('IndexError: list index is out of range')
        if index < 0:#return the head node element
            index = 0
            return self._head.element
        elif index >= self._length:#return tail
            index = self._length
            return self._tail.element
        #we have to find the element 
        current_index = 0
        current_node = self._head
        while current_node != None and current_index != index:
            current_node = current_node.next_node
            current_index = current_index + 1
        if current_node == None:
            raise IndexError('IndexError: list index is out of range')
        return current_node.element
        
    def __iter__(self):
        start_node = self._head
        while start_node != None:
            yield start_node.element
            start_node = start_node.next_node
    #
    def __len__(self):
        return self._length
    
    def __str__(self):
        return '%s has %s element(s)' % (self.__class__.__name__, str(self._length))
    
    def remove_duplicates(self):
        '''
        Removes nodes with duplicate elements. We build the corrected list.
        '''
        if self.length < 2:
            return
        last_node_of_corrected_list = self._head
        running_node = self._head
        current_node = last_node_of_corrected_list.next_node
        
        while current_node != None:#all the way until end
            running_node = self._head
            while running_node != current_node:
                if running_node.element == current_node.element: #duplicate
                    nxt_of_duplicate = current_node.next_node
                    last_node_of_corrected_list.next_node = nxt_of_duplicate
                    self._length = self._length - 1 
                    current_node = nxt_of_duplicate
                    break
                running_node = running_node.next_node
                
            if running_node == current_node:# current node did not have a duplicate element in the list seen so far
                last_node_of_corrected_list = current_node
                current_node = current_node.next_node
        
        self._tail = last_node_of_corrected_list    


'''
Created on Oct 28, 2015

@author: hari
'''
from __future__ import print_function
from builtins import range
import unittest
from pycoils.sets.disjoint_sets_with_union import DisjointSetWithUnion

class DisjointSet_With_Union_Test_init(unittest.TestCase):
    def setUp(self):
        pass
    
    def test_init(self):
        self._disjointset = DisjointSetWithUnion()
        
    def tearDown(self):
        self._disjointset = None

class DisjointSet_With_Union_Test_Size_Of_Empty_Set(unittest.TestCase):
    def setUp(self):
        pass
    
    def test_size_of_empty_set(self):
        self._disjointset = DisjointSetWithUnion()
        expected_size = 0
        self.assertEqual(self._disjointset.size, expected_size, 'Empty disjoint set size was not 0')
        
    def tearDown(self):
        self._disjointset = None

class DisjointSet_With_Union_Test_Find_with_Empty_Set(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
    
    def test_find_on_empty_set(self):
        self.assertEqual(self._disjointset.find(-1), None, 'Empty disjoint set find must return None')
        
    def tearDown(self):
        self._disjointset = None

class DisjointSet_With_Union_Test_Make_Set_with_None(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
    
    def test_make_set_with_none(self):
        self._disjointset.make_set(None)
        
    def tearDown(self):
        self._disjointset = None

class DisjointSet_With_Union_Test_Size_with_Single_Item(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
    
    def test_make_set_with_single_item(self):
        item = -1
        self._disjointset.make_set(item)
        expected_size = 1
        self.assertEqual(self._disjointset.size, expected_size, 'Singleton disjoint set size was not 1')
        
    def tearDown(self):
        self._disjointset = None

class DisjointSet_With_Union_Test_Make_Set_with_Single_Item(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
    
    def test_make_set_with_single_item(self):
        item = -1
        self._disjointset.make_set(item)
        self.assertEqual(self._disjointset.find(item), item, 'singleton disjoing set find subsequent to make set failed')
        
    def tearDown(self):
        self._disjointset = None

class DisjointSet_With_Union_Test_Make_Set_with_Multiple_Items(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
    
    def test_make_set_with_multiple_items(self):
        items = [1, 2, 3, 4, 5, 6, 7]
        for item in items:
            self._disjointset.make_set(item)
        for item in items:
            self.assertEqual(self._disjointset.find(item), item, 'singleton disjoint set find subsequent to make set failed')
        
    def tearDown(self):
        self._disjointset = None        

class DisjointSet_Test_Union_With_None(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
    
    def test_union_with_none(self):
        self.assertEqual(self._disjointset.union(item_1 = None, item_2 = None), None,
                                                'disjoint union with none,none failed')
    def test_union_with_none_and_value(self):
        self.assertEqual(self._disjointset.union(item_1 = None, item_2 = 'Some Value'), None,
                                                'disjoint union(None, some value) failed')
    def test_union_with_value_and_none(self):
        self.assertEqual(self._disjointset.union(item_1 = 'Some value', item_2 = None), None,
                                                'disjoint union(some value, None) failed')    
    def tearDown(self):
        self._disjointset = None

class DisjointSet_Test_Union_Towards_Single_Final_Set(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
        self._items = [1, 2, 3, 4, 5, 6, 7]
        for item in self._items:
            self._disjointset.make_set(item)
            
    def test_union_towards_single_final_set(self):
        items_count = len(self._items)
        set = None
        for index in range(0 , items_count - 1):
            set = self._disjointset.union(self._items[index], self._items[index + 1])
            self.assertEqual(self._disjointset.find(self._items[index]), set,
                              'Disjoint union-find failed while progressing to a single final set')
            self.assertEqual(self._disjointset.find(self._items[index + 1]), set,
                              'Disjoint union-find failed while progressing to a single final set')
        
    def tearDown(self):
        self._disjointset = None
        
class DisjointSet_Test_Union_Towards_Multiple_Final_Sets(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
        self._positive_items_1 = [1, 2, 3, 4, 5, 6, 7, 8]
        self._negative_items_1 = [-1, -2, -3, -4, -5]
        
        self._all_items =  [-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8]
        for item in self._all_items:
            self._disjointset.make_set(item)
            
    def test_union_towards_multiple_final_set(self):
        negative_items_count = len(self._negative_items_1)
        for index in range(0 , negative_items_count - 1):
            set = self._disjointset.union(self._negative_items_1[index], self._negative_items_1[index + 1])
            
        positive_items_count = len(self._positive_items_1)
        for index in range(0 , positive_items_count - 1):
            set = self._disjointset.union(self._positive_items_1[index], self._positive_items_1[index + 1])
        
        #Now there should be 3 sets with representations by -1, 0 and 1
        positive_representative_item = 1
        negative_representative_item = -1
        zero = 0
        
        for item in self._positive_items_1:
            self.assertEqual(self._disjointset.find(item), positive_representative_item,
                                            'Find failed after union towards multiple final sets.')
        
        for item in self._negative_items_1:
            self.assertEqual(self._disjointset.find(item), negative_representative_item,
                                            'Find failed after union towards multiple final sets.')
        
        self.assertEqual(self._disjointset.find(0), zero,
                                            'Find failed after union towards multiple final sets.')
        
    def tearDown(self):
        self._disjointset = None

class DisjointSet_With_Union_Test_Iter_On_Empty_Set(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
    
    def test_iter_on_empty_set(self):
        list_of_tuples = []
        for item_and_set in self._disjointset:
            list_of_tuples.append(item_and_set)
        self.assertEqual(len(list_of_tuples), 0, 'Iterating over empty set cannot yield anything')
        
    def tearDown(self):
        self._disjointset = None        

class DisjointSet_With_Union_Test_Iter_On_Singleton_Set(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
        self._item = 1
        self._disjointset.make_set(self._item)
        
    def test_iter_on_singleton_set(self):
        list_of_tuples = []
        for item_and_set in self._disjointset:
            list_of_tuples.append(item_and_set)
        self.assertEqual(len(list_of_tuples), 1, 'Iterating over (item, set) pairs in a singleton does not add up')
        
    def tearDown(self):
        self._disjointset = None


class DisjointSet_Test_Iter_After_Union_Towards_Multiple_Final_Sets(unittest.TestCase):
    def setUp(self):
        self._disjointset = DisjointSetWithUnion()
        self._positive_items_1 = [1, 2, 3, 4, 5, 6, 7, 8]
        self._negative_items_1 = [-1, -2, -3, -4, -5]
        
        self._all_items =  [-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8]
        for item in self._all_items:
            self._disjointset.make_set(item)
        
        negative_items_count = len(self._negative_items_1)
        for index in range(0 , negative_items_count - 1):
            set = self._disjointset.union(self._negative_items_1[index], self._negative_items_1[index + 1])
            
        positive_items_count = len(self._positive_items_1)
        for index in range(0 , positive_items_count - 1):
            set = self._disjointset.union(self._positive_items_1[index], self._positive_items_1[index + 1])
        #Now there should be 3 sets with representations by -1, 0 and 1
            
    def test_iter_after_union_towards_multiple_final_set(self):
        
        positive_representative_item = 1
        negative_representative_item = -1
        zero = 0
        
        for item, set_of_item in self._disjointset:
            if item == 0:
                self.assertEqual(set_of_item, zero, 'Iterating over (item, set) yields wrong set membership')
            elif item > 0:
                self.assertEqual(set_of_item, positive_representative_item, 'Iterating over (item, set) yields wrong set membership')
            else:
                self.assertEqual(set_of_item, negative_representative_item, 'Iterating over (item, set) yields wrong set membership')
                
    def tearDown(self):
        self._disjointset = None
        
if __name__ == '__main__':
    print('Disjoint sets with union tests')
    unittest.main()


'''
Created on Aug 14, 2015
@author: topcat
'''
from __future__ import print_function
import unittest
from pycoils.stackNqueues.stack import Stack

class Test_Empty_Stack_Initialization(unittest.TestCase):
    
    def setUp(self):
        self._stack = Stack()
    
    def test_stack_size_is_zero(self):
        self.assertEqual(self._stack.size, 0, 'Empty stack size must be 0')
        
    def tearDown(self):
        del self._stack

class Test_Empty_Stack_Pop(unittest.TestCase):
    
    def setUp(self):
        self._stack = Stack()
    
    def test_empty_stack_pop(self):
        self.assertEqual(self._stack.pop(), None, 'Empty stack must pop None')
                
    def tearDown(self):
        del self._stack

class Test_Empty_Stack_Top(unittest.TestCase):
        
    def setUp(self):
        self._stack = Stack()
    
    def test_empty_stack_top(self):
        self.assertEqual(self._stack.top, None, 'Empty Stact top must be None')
                
    def tearDown(self):
        del self._stack


class Test_Predictable_Stack_Size_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._stack = Stack()
        self._stack.push(1)
        self._stack.push(2)
        self._stack.push(3)
        self._stack.push(4)
    
    def test_stack_size_with_4_elements(self):
        self.assertEqual(self._stack.size, 4, 'NA')
                
    def tearDown(self):
        del self._stack

class Test_Predictable_Stack_Top_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._stack = Stack()
        self._stack.push(1)
        self._stack.push(2)
        self._stack.push(3)
        self._stack.push(4)
    
    def test_stack_top_with_4_elements(self):
        self.assertEqual(self._stack.top, 4, 'NA')
                
    def tearDown(self):
        del self._stack

class Test_Pop_To_Empty_With_Predictable_Stack_Of_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._stack = Stack()
        self._stack.push(1)
        self._stack.push(2)
        self._stack.push(3)
        self._stack.push(4)
    
    def test_pop_to_empty_stack_with_4_elements(self):
        self.assertEqual(self._stack.size, 4, 'NA')
        
        elements = [4, 3, 2, 1]
        for element in elements:#pop 4 times
            self.assertEqual(self._stack.pop(), element)
        
        self.assertEqual(self._stack.pop(), None, 'Empty stack must return None')
        self.assertEqual(self._stack.size, 0, 'NA')
            
    def tearDown(self):
        del self._stack

class Test_Iterator_With_Stack_Of_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._stack = Stack()
        self._stack.push(1)
        self._stack.push(2)
        self._stack.push(3)
        self._stack.push(4)
    
    def test_iterator_with_stack_with_4_elements(self):
        elements = [4, 3, 2, 1]
        for stack_element in self._stack:
            self.assertEqual(stack_element, elements[0], 'stack Iterator must match expected elements.')
            del elements[0]
            
    def tearDown(self):
        del self._stack
#Allows running as python run.
if __name__ == '__main__':
    print('stack Tests')
    unittest.main()


'''
Created on Nov 6, 2015
@author: hari
'''
from builtins import object
from pycoils.hashtables.HashTables import SeperateChainHashTable
import weakref

class InternStore(object):
    '''
    Several objects of a class can have the same internal state. Holding all these objects 
    in memory is a waste. InternStore holds all these same internal state objects to 
    refer to a single weak reference object. For this to work all the objects you send to the
    intern store must be hashable. Hash of the objects must return the same hash for the 
    same internal states. 
    '''
    def __init__(self):
        self._store = SeperateChainHashTable()
    
    def intern(self, obj):
        '''
        an object obj is checked against the intern store and if there is an existing weakref
        with hash = hash(obj) then that instance is returned. If no entry is found a weak 
        reference to obj is created and stored.
        '''
        if obj == None:
            return None
        
        class_of_obj = type(obj)
        map_of_class_to_weak_refs_of_obj = self._store.get(class_of_obj, default = None)
        if map_of_class_to_weak_refs_of_obj is None:
            map_of_class_to_weak_refs_of_obj = weakref.WeakValueDictionary()
            self._store[class_of_obj] = map_of_class_to_weak_refs_of_obj
        
        weak_ref_to_obj = map_of_class_to_weak_refs_of_obj.get(obj, default = None)
        strong_ref_obj = None 
        
        if weak_ref_to_obj is not None:
            strong_ref_obj = weak_ref_to_obj
            return strong_ref_obj
        else:
            map_of_class_to_weak_refs_of_obj[obj] = obj 
            strong_ref_obj = obj
            return strong_ref_obj


'''
Created on Oct 29, 2015

@author: hari
'''
from __future__ import print_function
from builtins import str
from pycoils.sets.disjoint_sets_with_union import DisjointSetWithUnion
if __name__ == '__main__':
    integers = [-5, -3, 0, 2, 10]
    disjoint_set = DisjointSetWithUnion()
    
    for integer in integers:
        disjoint_set.make_set(integer)
        
    item_to_find = -3
    set_rep = disjoint_set.find(item_to_find)
    print('Item %d belongs to set represented by %d' % (item_to_find, set_rep))
    
    #join set represented by -5 to the set represented by -3
    print('Joining sets represented by -5 and -3')
    disjoint_set.union(-5, -3)
    
    print('Joining sets represented by 2 and 10')
    disjoint_set.union(2, 10)
    
    
    item_to_find = -3
    set_rep = disjoint_set.find(item_to_find)
    print('Item %d belongs to set represented by %d' % (item_to_find, set_rep))
    
    item_to_find = 10
    set_rep = disjoint_set.find(item_to_find)
    print('Item %d belongs to set represented by %d' % (item_to_find, set_rep))
    
    print('Iterating over the Disjoint set to get (item, set) pairs')
    for item, set_to_which_item_belongs in disjoint_set:
        print('Item %s belongs to set %s' % (str(item), str(set_to_which_item_belongs)))


'''
Created on Oct 19, 2015
@author: hari
'''
from __future__ import print_function
from unittest import TestCase
from pycoils.trees.bst import BinarySearchTree
import unittest
#
# Preorder traversal
#
class BST_Test_Preorder_Traversal_with_empty_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
    
    def test_preorder_traversal_with_empty_tree(self):
        pre_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = True, want_in_order = False, want_post_order = False):
            pre_order_elements.append(element)
        self.assertEqual(0, len(pre_order_elements), 'Pre order traversal on empty tree must yield no elements')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Preorder_Traversal_with_Single_Node_Tree(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 10, obj = 10)
    
    def test_preorder_traversal_with_single_node_tree(self):
        pre_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = True, want_in_order = False, want_post_order = False):
            pre_order_elements.append(element)
        self.assertEqual(1, len(pre_order_elements), 'Pre order traversal on single node tree must yield no elements')
        self.assertEqual(pre_order_elements, [10], 'Pre order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Preorder_Traversal_with_10_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
    def test_preorder_traversal_with_10_node_tree(self):
        pre_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = True, want_in_order = False, want_post_order = False):
            pre_order_elements.append(element)
        self.assertEqual(10, len(pre_order_elements), 'Pre order traversal on 10 node tree must yield 10 elements')
        self.assertEqual(pre_order_elements, [5, 2, 1, 3, 4, 8, 7, 6, 9, 10], 'Pre order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None        
#
# Postorder traversal
#
class BST_Test_Postorder_Traversal_with_empty_Tree(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
    
    def test_postorder_traversal_with_empty_tree(self):
        post_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = False, want_post_order = True):
            post_order_elements.append(element)
        self.assertEqual(0, len(post_order_elements), 'Post order traversal on empty tree must yield no elements')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Postorder_Traversal_with_Single_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 10, obj = 10)
    
    def test_postorder_traversal_with_single_node_tree(self):
        post_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = False, want_post_order = True):
            post_order_elements.append(element)
        self.assertEqual(1, len(post_order_elements), 'Post order traversal on single node tree must yield one elements')
        self.assertEqual(post_order_elements, [10], 'Post order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Postorder_Traversal_with_10_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
    def test_postorder_traversal_with_10_node_tree(self):
        post_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = False, want_post_order = True):
            post_order_elements.append(element)
        self.assertEqual(10, len(post_order_elements), 'Post order traversal on 10 node tree must yield 10 elements')
        self.assertEqual(post_order_elements, [1, 4, 3, 2, 6, 7, 10, 9, 8, 5], 'Post order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None
#
# Inorder traversal
#
class BST_Test_Inorder_Traversal_with_empty_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
    
    def test_inorder_traversal_with_empty_tree(self):
        in_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = True, want_post_order = False):
            in_order_elements.append(element)
        self.assertEqual(0, len(in_order_elements), 'In order traversal on empty tree must yield no elements')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Inorder_Traversal_with_Single_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 10, obj = 10)
    
    def test_inorder_traversal_with_single_node_tree(self):
        in_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = True, want_post_order = False):
            in_order_elements.append(element)
        self.assertEqual(1, len(in_order_elements), 'In order traversal on single node tree must yield one elements')
        self.assertEqual(in_order_elements, [10], 'In order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Inorder_Traversal_with_10_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
    
    def test_inorder_traversal_with_10_node_tree(self):
        in_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = True, want_post_order = False):
            in_order_elements.append(element)
        self.assertEqual(10, len(in_order_elements), 'In order traversal on 10 node tree must yield 10 elements')
        self.assertEqual(in_order_elements, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 'In order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None
        
class BST_Test_All_Traversals_Consecutively_For_Tree_Integirty_With_10_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
    def test_traversal_consecutively_with_10_node_tree(self):
        #inorder
        in_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = True, want_post_order = False):
            in_order_elements.append(element)
        self.assertEqual(10, len(in_order_elements), 'In order traversal on 10 node tree must yield 10 elements')
        self.assertEqual(in_order_elements, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 'In order traversal did not yield expected elements in order')
        
        #postorder
        post_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = False, want_in_order = False, want_post_order = True):
            post_order_elements.append(element)
        self.assertEqual(10, len(post_order_elements), 'Post order traversal on 10 node tree must yield 10 elements')
        self.assertEqual(post_order_elements, [1, 4, 3, 2, 6, 7, 10, 9, 8, 5], 'Post order traversal did not yield expected elements in order')
    
        pre_order_elements = []
        for key, element in self._bst.traversal(want_pre_order = True, want_in_order = False, want_post_order = False):
            pre_order_elements.append(element)
        self.assertEqual(10, len(pre_order_elements), 'Pre order traversal on 10 node tree must yield 10 elements')
        self.assertEqual(pre_order_elements, [5, 2, 1, 3, 4, 8, 7, 6, 9, 10], 'Pre order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None
#
# Inorder traversal with stack
#
class BST_Test_Inorder_Traversal_Using_Stack_with_empty_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
    
    def test_inorder_traversal_with_empty_tree(self):
        in_order_elements = []
        for key, element in self._bst.inorder_traversal_with_stack():
            in_order_elements.append(element)
        self.assertEqual(0, len(in_order_elements), 'In order traversal on empty tree must yield no elements')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Inorder_Traversal_with_Stack_On_Single_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 10, obj = 10)
    
    def test_inorder_traversal_with_single_node_tree(self):
        in_order_elements = []
        for key, element in self._bst.inorder_traversal_with_stack():
            in_order_elements.append(element)
        self.assertEqual(1, len(in_order_elements), 'In order traversal on single node tree must yield one elements')
        self.assertEqual(in_order_elements, [10], 'In order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Inorder_Traversal_with_Stack_on_10_Node_Tree(TestCase):
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
    
    def test_inorder_traversal_with_10_node_tree(self):
        in_order_elements = []
        for key, element in self._bst.inorder_traversal_with_stack():
            in_order_elements.append(element)
        self.assertEqual(10, len(in_order_elements), 'In order traversal on 10 node tree must yield 10 elements')
        self.assertEqual(in_order_elements, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 'In order traversal did not yield expected elements in order')
    
    def tearDown(self):
        self._bst = None
                          
#Allows running as python run.
if __name__ == '__main__':
    print('BST Traversal test')
    unittest.main()


'''
Created on Aug 14, 2015
@author: topcat
'''
from builtins import object
class Stack(object):
    """
    A stack implementation in Python. Uses Python list to hold stack items.
    """
    def __init__(self):
        self._list = []
    
    def push(self, item):
        self._list.append(item)
    
    def pop(self):
        if len(self._list) == 0:
            return None
            
        index = len(self._list) - 1
        item = self._list[index]
        del self._list[index]
        return item
    
    def __len__(self):
        return len(self._list)
    
    @property
    def top(self):
        if len(self._list) > 0:
            return self._list[len(self._list) - 1]
        else:
            return None
    @property
    def size(self):
        return len(self._list)
    
    def __iter__(self):
        return self
    
    def __next__(self):
        stack_item = self.pop()
        if stack_item == None:
            raise StopIteration
        return stack_item 
    
    def __str__(self):
        return '%s size [%s] items %s' % (self.__class__.__name__, len(self), self._list)


'''
Created on Oct 25, 2015
@author: hari
'''
from __future__ import print_function
from builtins import str
from pycoils.stackNqueues.queue import Queue
if __name__ == '__main__':
    q = Queue()
    print('Q size is %s' % str(q.size))
    #enqueue
    print('Adding 1, 2, 3, 4, 5, 6 elements to queue')
    q.enqueue(1)
    q.enqueue(2)
    q.enqueue(3)
    q.enqueue(4)
    q.enqueue(5)
    q.enqueue(6)
    #front
    print('Q front is %s' % str(q.front))
    
    #iterate and dequeue each element
    print('Iterating and removing items')
    for item in q:
        print(item)



'''
Created on Oct 25, 2015
@author: hari
'''
from __future__ import print_function
from builtins import str
from pycoils.stackNqueues.stack import Stack

if __name__ == '__main__':
    stack = Stack()
    # count items
    print('stack items count: %s' % str(stack.size))
    #Push items
    stack.push(item = 1)
    stack.push(item = 2)
    stack.push(item = 3)
    stack.push(item = 4)
    stack.push(item = 5)
    stack.push(item = 6)
    #top
    print('stack top is %s' % str(stack.top))
    #pop
    popped = stack.pop()
    print('Popped item is %s' % str(popped))
    #iterate over the stack popping items one by one
    print('Iterate over stack popping items')
    for item in stack:
        print('item %s' % str(item))
        



'''
Created on Oct 24, 2015
@author: hari
'''
from __future__ import print_function
from builtins import range
import unittest
from pycoils.lists.doubly_linked_list import DoublyLinkedList
from random import randint

class DoublyDoublyLinkedList_Test_Count_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_count_of_empty_list(self):
        self.assertEqual(self._doubly_linkedList.length, 0, 'Empty List length must be 0')
    
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Head_n_Tail_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_count_of_empty_list(self):
        self.assertEqual(self._doubly_linkedList.head, None, 'Head of 0 element list must be None')
        self.assertEqual(self._doubly_linkedList.tail, None, 'Tail of 0 element list must be None')
    
    def tearDown(self):
        self._doubly_linkedList = None
    
        
class DoublyDoublyLinkedList_Test_Count_With_4_Predictable_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
    
    def test_count_of_list(self):
        self.assertEqual(self._doubly_linkedList.length, 4, 'Empty List length must be 4')
    
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Non_Existing_Element_From_0_Element_List(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_count_of_list(self):
        self._doubly_linkedList.remove(-1)
        self.assertEqual(self._doubly_linkedList.length, 0, 'Length must be 0 for empty list')
    
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Non_Existing_Element_From_4_Element_List(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
    
    def test_count_of_list(self):
        self._doubly_linkedList.remove(-1)
        self.assertEqual(self._doubly_linkedList.length, 4, 'Length not valid after trying to remove non-existing element')

    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Middle_Element_From_4_Element_List(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
    
    def test_count_of_list(self):
        self._doubly_linkedList.remove(3)
        self.assertEqual(self._doubly_linkedList.length, 3, 'Length not valid after trying to remove non-existin element')

    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_IndexOf_Element_In_Empty_List(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_index(self):
        index = self._doubly_linkedList.index(8)
        self.assertEqual(index, -1, 'Non existing element has index -1')

    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_IndexOf_Elements_In_4_Element_List(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
    
    def test_index(self):
        for element in range(1, 5):
            index = self._doubly_linkedList.index(element)
            self.assertEqual(index + 1, element, 'index failed for element')

    def tearDown(self):
        self._doubly_linkedList = None    

class DoublyDoublyLinkedList_Test_Operations_On_List_with_1000_Random_Elements(unittest.TestCase):
    def setUp(self):
        self._linked_list = DoublyLinkedList()
        self._shadow_list = []
        for i in range(10):
            random_element = randint(0, 1000)
            self._linked_list.append(random_element)
            self._shadow_list.append(random_element)#will keep track of count and index 
    
    def test_count_index_remove_index_count_on_each_removal(self):
        """
        Check count, index before and after each predictable removal operation 
        """
        for i in self._shadow_list[:]:
            self.assertEqual(self._linked_list.length, len(self._shadow_list))
            
            self.assertEqual(self._linked_list.index(i), self._shadow_list.index(i))
            
            self._shadow_list.remove(i)
            self._linked_list.remove(i)
            
            try:
                self._shadow_list.index(i)
                self.assertEqual(self._linked_list.index(i), self._shadow_list.index(i))
            except ValueError:
                self.assertEqual(self._linked_list.index(i), -1)
            
            self.assertEqual(self._linked_list.length, len(self._shadow_list))

class DoublyDoublyLinkedList_Test_extend_With_Predictable_Elements(unittest.TestCase):
    '''
    Test the extend operation of LinkedList. 
    a) Create 2 lists. 
    b) Add the second list to the end of the first list. 
    c) Check the indices of the newly added elements.
    '''
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
        #create a second linkedlist with 5, 6, 7
        self._secondList = DoublyLinkedList()
        self._secondList.append(5)
        self._secondList.append(6)
        self._secondList.append(7)
        
    def test_extend_of_list(self):
        self.assertEqual(self._doubly_linkedList.length, 4, 'Original list before extend must be 4')
        count_before_extend = self._doubly_linkedList.length
        next_index_will_be = count_before_extend
        self._doubly_linkedList.extend(self._secondList)
        #check new length
        self.assertEqual(self._doubly_linkedList.length, count_before_extend + self._secondList.length, 'New length after extend must add up.')
        #check if the elements of shadow list are present
        for element in self._secondList:
            self.assertEqual(self._doubly_linkedList.index(element), next_index_will_be
                             , 'Indices of new elements after extend must add up')
            next_index_will_be += 1
    
    def tearDown(self):
        self._doubly_linkedList = None
        self._secondList = None

class DoublyDoublyLinkedList_Test_Check_Tail_On_Remove_In_4_Element_List(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
    
    def test_tail_node_on_remove(self):
        self._doubly_linkedList.remove(1)#removed head. Tail must be same node 
        self.assertEqual(self._doubly_linkedList.tail, 4)
        #Now list is [2, 3, 4]
        self._doubly_linkedList.remove(4)#tail should next be 3
        self.assertEqual(self._doubly_linkedList.tail, 3, 'Tail must be adjusted on deletion of last node')
        #Now list is [2, 3]
        self._doubly_linkedList.remove(2)#tail should next be 3
        self.assertEqual(self._doubly_linkedList.tail, 3, 'Tail must be adjusted on deletion of last node')
        #Now list is [3]
        self._doubly_linkedList.remove(3)#tail should next be None
        self.assertEqual(self._doubly_linkedList.tail, None, 'Tail must be None on empty list')
    def tearDown(self):
        self._doubly_linkedList = None    

class DoublyDoublyLinkedList_Test_Check_Head_On_Remove_In_4_Element_List(unittest.TestCase):

    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
    
    def test_tail_node_on_remove(self):
        #list is [1, 2, 3, 4]
        self._doubly_linkedList.remove(3)#removed head. Head must be same node 
        self.assertEqual(self._doubly_linkedList.head, 1)
        #Now list is [1, 2, 4]
        self._doubly_linkedList.remove(1)#head should next be 2
        self.assertEqual(self._doubly_linkedList.head, 2, 'head must be adjusted on deletion of first node')
        #Now list is [2, 4]
        self._doubly_linkedList.remove(2)#head should next be 4
        self.assertEqual(self._doubly_linkedList.head, 4, 'head must be adjusted on deletion of first node')
        #Now list is [4]
        self._doubly_linkedList.remove(4)#head should next be None
        self.assertEqual(self._doubly_linkedList.head, None, 'Head must be None on empty list')
        
    def tearDown(self):
        self._doubly_linkedList = None    

class DoublyDoublyLinkedList_Test_Insert_At_0_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_insert_at_0_of_empty_list(self):
        self._doubly_linkedList.insert_at(0, 20)
        self.assertEqual(self._doubly_linkedList.length, 1, 'list length must be 1 after inserting to empty list')
        self.assertEqual(self._doubly_linkedList.head, self._doubly_linkedList.tail, 'list length must be same node after inserting to empty list')
    
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Insert_At_Negative_Index_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_insert_at_negative_index_of_empty_list(self):
        self._doubly_linkedList.insert_at(-2, 20)
        self.assertEqual(self._doubly_linkedList.length, 1, 'list length must be 1 after inserting to empty list at negative index')
        self.assertEqual(self._doubly_linkedList._head, self._doubly_linkedList._tail, 'list length must be same node after inserting to empty list at negative index')
    
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Insert_At_Index_Larger_Than_Length_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_insert_at_negative_index_of_empty_list(self):
        self._doubly_linkedList.insert_at(100, 20)
        self.assertEqual(self._doubly_linkedList.length, 1, 'list length must be 1 after inserting to empty list at huge index')
        self.assertEqual(self._doubly_linkedList.head, self._doubly_linkedList.tail, 'list length must be same node after inserting to empty list at huge index')
    
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Insert_At_Index_Larger_Than_Length_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.append(4)
        
    def test_insert_at_negative_index_of_empty_list(self):
        expected_length = self._doubly_linkedList.length + 1
        element_to_insert = 20
        self._doubly_linkedList.insert_at(100, element_to_insert)
        self.assertEqual(self._doubly_linkedList.length, expected_length, 'list length did not tally after insert at huge index')
        self.assertEqual(self._doubly_linkedList.tail, element_to_insert, 'list tail must be same as new element inserted at huge index')
        self.assertEqual(self._doubly_linkedList.index(20), expected_length - 1, 'index of new element did not add up')
    
    def tearDown(self):
        self._doubly_linkedList = None


class DoublyDoublyLinkedList_Test_Insert_At_Index_In_Middle_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
        self._doubly_linkedList.append(2)
        self._doubly_linkedList.append(4)
        self._doubly_linkedList.append(5)
        
    def test_insert_at_negative_index_of_empty_list(self):
        expected_length = self._doubly_linkedList.length + 1
        element_to_insert = 3
        insert_at_index = 2
        self._doubly_linkedList.insert_at(insert_at_index, element_to_insert)# list will become 1, 2, 3, 4, 5
        self.assertEqual(self._doubly_linkedList.length, expected_length, 'list length did not tally after insert in the middle')
        self.assertEqual(self._doubly_linkedList.index(3), insert_at_index, 'index of new element is not where we insert it at!')
        
        #iterate and check
        expected_list = []
        for element in self._doubly_linkedList:
            expected_list.append(element)
        self.assertEqual(expected_list, [1, 2, 3, 4, 5], 'List was not same as expected after insert At index')
    
    def tearDown(self):
        self._doubly_linkedList = None
#
# Tests for remover duplicates. Need to check head and tail refs directly rather than elements.
#
class DoublyDoublyLinkedList_Test_Remove_Duplicates_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_remove_duplicate_of_empty_list(self):
        self._doubly_linkedList.remove_duplicates()
        
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Duplicates_With_1_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        element_to_append = 1
        self._doubly_linkedList.append(element_to_append)
        
    def test_remove_duplicate_of_list_with_1_element(self):
        old_head = self._doubly_linkedList._head
        expected_tail = old_head
        
        self._doubly_linkedList.remove_duplicates()
        expected_length = 1
        self.assertEqual(self._doubly_linkedList.length, expected_length, 'list length did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._head, old_head, 'list head did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._tail, expected_tail, 'list tail did not add up after removing duplicates')
        
        
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Duplicates_With_2_Identical_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        element_to_append = 1
        self._doubly_linkedList.append(element_to_append)
        self._doubly_linkedList.append(element_to_append)
        
    def test_remove_duplicates_of_list_with_2_identical_elements(self):
        old_head = self._doubly_linkedList._head
        expected_tail = old_head
        
        self._doubly_linkedList.remove_duplicates()
        expected_length = 1
        self.assertEqual(self._doubly_linkedList.length, expected_length, 'list length did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._head, old_head, 'list head did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._tail, expected_tail, 'list tail did not add up after removing duplicates')
        
    def tearDown(self):
        self._doubly_linkedList = None
        
class DoublyDoublyLinkedList_Test_Remove_Duplicates_With_Duplicates_At_Head(unittest.TestCase):
    
    def setUp(self):
        #build [1, 1, 1, 2]
        self._doubly_linkedList = DoublyLinkedList()
        element_to_append = 1
        self._doubly_linkedList.extend([element_to_append, element_to_append, element_to_append])
        tail_element = 2
        self._doubly_linkedList.append(tail_element)
        
    def test_remove_duplicates_of_list_with_2_identical_elements(self):
        expected_head = self._doubly_linkedList._head
        expected_tail = self._doubly_linkedList._tail
        
        self._doubly_linkedList.remove_duplicates()
        expected_length = 2
        self.assertEqual(self._doubly_linkedList._length, expected_length, 'list length did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._head, expected_head, 'list head did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._tail, expected_tail, 'list tail did not add up after removing duplicates')
        
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Duplicates_With_Duplicates_At_Tail(unittest.TestCase):
    
    def setUp(self):
        #build [2, 1, 1, 1]
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(2)
        element_to_append = 1
        self._doubly_linkedList.extend([element_to_append, element_to_append, element_to_append])
        
    def test_remove_duplicates_of_list_with_2_identical_elements(self):
        expected_head = self._doubly_linkedList._head
        expected_tail = self._doubly_linkedList._head.next_node
        
        self._doubly_linkedList.remove_duplicates()
        expected_length = 2
        self.assertEqual(self._doubly_linkedList.length, expected_length, 'list length did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._head, expected_head, 'list head did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._tail, expected_tail, 'list tail did not add up after removing duplicates')
        
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Duplicates_With_Duplicates_At_Middle(unittest.TestCase):
    
    def setUp(self):
        #build [2, 1, 1, 1, 3]
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(2)
        element_to_append = 1
        self._doubly_linkedList.extend([element_to_append, element_to_append, element_to_append])
        self._doubly_linkedList.append(3)
        
    def test_remove_duplicates_of_list_with_2_duplicates_in_Middle(self):
        expected_head = self._doubly_linkedList._head
        expected_tail = self._doubly_linkedList._tail
        
        self._doubly_linkedList.remove_duplicates()
        expected_length = 3
        self.assertEqual(self._doubly_linkedList.length, expected_length, 'list length did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._head, expected_head, 'list head did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._tail, expected_tail, 'list tail did not add up after removing duplicates')
        
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyDoublyLinkedList_Test_Remove_Duplicates_With_Recurring_Duplicates(unittest.TestCase):
    
    def setUp(self):
        #build [2, 1, 1, 1, 3, 1, 1, 1, 4]
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(2)
        element_to_append = 1
        self._doubly_linkedList.extend([element_to_append, element_to_append, element_to_append])
        self._doubly_linkedList.append(3)
        self._doubly_linkedList.extend([element_to_append, element_to_append, element_to_append])
        self._doubly_linkedList.append(4)
        
    def test_remove_duplicates_of_list_with_recurring_duplicates(self):
        expected_head = self._doubly_linkedList._head
        expected_tail = self._doubly_linkedList._tail
        
        self._doubly_linkedList.remove_duplicates()
        expected_length = 4
        self.assertEqual(self._doubly_linkedList.length, expected_length, 'list length did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._head, expected_head, 'list head did not add up after removing duplicates')
        self.assertEqual(self._doubly_linkedList._tail, expected_tail, 'list tail did not add up after removing duplicates')
        
    def tearDown(self):
        self._doubly_linkedList = None
#
# Test Reverse Iteration
#
class DoublyDoublyLinkedList_Test_Reverse_Iteration_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_reverse_iteration_of_empty_list(self):
        reverse_iter_of_list = [] 
        for element in reversed(self._doubly_linkedList):
            reverse_iter_of_list.append(element)

        self.assertEqual(len(reverse_iter_of_list), 0, 'Reverse iteration on empty list gives non-empty values!')
        
    def tearDown(self):
        self._doubly_linkedList = None
        
class DoublyLinkedList_Test_Reverse_Iteration_With_1_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._doubly_linkedList.append(1)
    
    def test_reverse_iteration_of_1_element_list(self):
        expected_reverse_iter_list  = [1] 
        reverse_iter_of_list = [] 
        for element in reversed(self._doubly_linkedList):
            reverse_iter_of_list.append(element)

        self.assertEqual(reverse_iter_of_list, expected_reverse_iter_list, 'reverse iteration did not yield expected list')
        
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyLinkedList_Test_Reverse_Iteration_With_multiple_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        extend_elements = [1, 2, 3, 4]
        self._doubly_linkedList.extend(extend_elements)
    
    def test_reverse_iteration_with_multiple_elements(self):
        expected_reverse_iter_list  = [4, 3, 2, 1] 
        reverse_iter_of_list = [] 
        for element in reversed(self._doubly_linkedList):
            reverse_iter_of_list.append(element)

        self.assertEqual(reverse_iter_of_list, expected_reverse_iter_list, 'reverse iteration did not yield expected list')
        
    def tearDown(self):
        self._doubly_linkedList = None
#
# Test __getitem__
#
class DoublyDoublyLinkedList_Test_get_item_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
    
    def test_get_item_of_empty_list(self):
        for index in [-1, 0, 1]:
            with self.assertRaises(IndexError):
                self._doubly_linkedList[index]
        
    def tearDown(self):
        self._doubly_linkedList = None
    
class DoublyLinkedList_Test_get_item_With_1_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        
    def test_reverse_iteration_of_1_element_list(self):
        element_to_add = 1
        self._doubly_linkedList.append(element_to_add)
        self.assertEqual(self._doubly_linkedList[0], element_to_add, 'list get item did not yield expected item')
        self.assertEqual(self._doubly_linkedList[-1], element_to_add, 'list get item did not yield expected item')
        self.assertEqual(self._doubly_linkedList[10], element_to_add, 'list get item did not yield expected item')
        
    def tearDown(self):
        self._doubly_linkedList = None

class DoublyLinkedList_Test_get_item_With_multiple_Elements(unittest.TestCase):
    
    def setUp(self):
        self._doubly_linkedList = DoublyLinkedList()
        self._extend_elements = [1, 2, 3, 4]
        self._doubly_linkedList.extend(self._extend_elements)
    
    def test_get_item_with_multiple_elements(self):
        for i in self._extend_elements:#loop through the items we added. item 1 will be at index 0, 2 at 1 and so on.
            self.assertEqual(self._doubly_linkedList[i - 1], i, 'list get item at index failed')
    
    def tearDown(self):
        self._doubly_linkedList = None
#Allows running as python run.
if __name__ == '__main__':
    print('Doubly Linked List tests')
    unittest.main()



'''
Created on Oct 23, 2015

@author: hari
'''
from builtins import range
import unittest
from pycoils.hashtables.hashbuckets import SplayedHashBucket

class SplayedHashBucket_TestCase_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._hash_bucket = SplayedHashBucket()
    
    def test_count_of_empty_list(self):
        self.assertEqual(len(self._hash_bucket), 0, 'Empty bucket length must be 0')
    
    def tearDown(self):
        self._hash_bucket = None

class SplayedHashBucket_TestCase_Len_With_4_Elements(unittest.TestCase):
    def setUp(self):
        self._hash_bucket = SplayedHashBucket()
        self._hash_bucket[1] = 1
        self._hash_bucket[2] = 2
        self._hash_bucket[3] = 3
        self._hash_bucket[4] = 4
        
    def test_len_of_empty_hash_bucket(self):
        self.assertEqual(len(self._hash_bucket), 4, 'Empty bucket length did not add up')
    
    def tearDown(self):
        self._hash_bucket = None

class SplayedHashBucket_TestCase_Get_Item_With_4_Elements(unittest.TestCase):
    def setUp(self):
        self._hash_bucket = SplayedHashBucket()
        self._hash_bucket[1] = 1
        self._hash_bucket[2] = 2
        self._hash_bucket[3] = 3
        self._hash_bucket[4] = 4
    
    def test_get_item_hash_bucket_with_default_return(self):
        for i in range(1, 5):
            self.assertEqual(self._hash_bucket.get(i), i, 'Hash bucket items did not match on retirieval')
    
    def test_get_item_hash_bucket(self):
        for i in range(1, 5):
            self.assertEqual(self._hash_bucket[i], i, 'Hash bucket items did not match on retirieval')

    def tearDown(self):
        self._hash_bucket = None
    
class SplayedHashBucket_TestCase_Get_Non_Existing_Item_With_4_Elements(unittest.TestCase):
    def setUp(self):
        self._hash_bucket = SplayedHashBucket()
    
    def test_get_non_exiting_item_of_hash_bucket_expect_None_as_default(self):
        self.assertEqual(self._hash_bucket.get(1), None, 
                                        'default value for non existing key was not used')
    
    def test_get_non_exiting_item_of_hash_bucket(self):
        with self.assertRaises(KeyError):
            self._hash_bucket[1]
        
    def test_get_non_exiting_item_of_hash_bucket_expect_Something_as_default(self):
        expected_default_return = "Default"
        self.assertEqual(self._hash_bucket.get(1, default = expected_default_return), expected_default_return, 
                                        'default value for non existing key was not used')
    def tearDown(self):
        self._hash_bucket = None

class SplayedHashBucket_TestCase_Delete_Item_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._hash_bucket = SplayedHashBucket()
        self._hash_bucket[1] = 1
        self._hash_bucket[2] = 2
        self._hash_bucket[3] = 3
        self._hash_bucket[4] = 4
        
    def test_delete_item_hash_bucket(self):
        for i in range(1, 5):
            self.assertEqual(self._hash_bucket[i], i, 'Hash bucket items did not match on retirieval')
            del self._hash_bucket[i]
            with self.assertRaises(KeyError):
                self._hash_bucket[i]
            
    def tearDown(self):
        self._hash_bucket = None

class SplayedHashBucket_TestCase_Set_Item_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._hash_bucket = SplayedHashBucket()
        self._hash_bucket[1] = 1
        self._hash_bucket[2] = 2
        self._hash_bucket[3] = 3
        self._hash_bucket[4] = 4
        
    def test_set_item_hash_bucket(self):
        for i in range(1, 5):
            self.assertEqual(self._hash_bucket[i], i, 'Hash bucket items did not match on retirieval')
            self._hash_bucket[i] = 5 + i
            self.assertEqual(self._hash_bucket[i], 5 + i, 'Hash bucket items did not match on retirieval')
            
    def tearDown(self):
        self._hash_bucket = None

class SplayedHashBucket_TestCase_Iter_Keys_With_4_Elements(unittest.TestCase):
    
    def setUp(self):
        self._hash_bucket = SplayedHashBucket()
        self._hash_bucket[1] = 1
        self._hash_bucket[2] = 2
        self._hash_bucket[3] = 3
        self._hash_bucket[4] = 4
        
    def test_set_item_hash_bucket(self):
        expected_key_list = [1, 2, 3, 4] #underlying ds for bucket is list. we inserted in this order. so we expect in this order.
        actual_key_list = []
        for key in self._hash_bucket:
            actual_key_list.append(key)
        self.assertEqual(expected_key_list, actual_key_list, 'Iterating over bucket keys did not match expected list of keys')
    def tearDown(self):
        self._hash_bucket = None
        
#Allows running as python run.
if __name__ == '__main__':
    unittest.main()



'''
Created on Oct 29, 2015
@author: hari
'''
from builtins import object
from pycoils.trees.heap import Heap
from functools import total_ordering

@total_ordering
class PriorityQueueEntry(object):
    '''
    An entry in the priority Q is made up of an item and its priority. priority can be anything but must
    be comparable. 
    '''
    __slots__ = 'item', 'priority'
    def __init__(self, item = None, priority = None):
        if item == None:#cannot waste slots
            raise ValueError('Priority Q entry item cannot be None')
        if priority == None:
            raise ValueError('Priority Q entry priority cannot be None')
        
        self.item = item
        self.priority = priority
    
    def __eq__(self, other_entry):
        if other_entry is None:
            return False
            
        return self.priority == other_entry.priority
            
    def __lt__(self, other_entry): 
        if other_entry is None:
            return False
        
        return self.priority < other_entry.priority

class PriorityQueue(object):
    '''
    Priority Q based on Heaps. You can set min heap of max heap. Default is Max Heap.
    1) Iterating yields (item, priority) tuples
    '''
    
    def __init__(self, reverse_priority = False):
        '''
        creates a priority queue such that priority 10 is processed first than priority 2. If you want
        to reverse this set reverse_priority so that item with priority 2 is processed first 
        '''
        self._heap = Heap(min_heap =  reverse_priority)#create the heap as needed. Default Max heap
    
    def add_item(self, item = None, priority = None):
        if item == None:
            return #we are not wasting time
        
        self._heap.add_item(PriorityQueueEntry(item = item, priority = priority))
    
    def get(self):
        priority_q_entry = self._heap.get()
        if priority_q_entry == None:
            return None
        return priority_q_entry.item
    
    def __len__(self):
        return self._heap.heap_size
    
    def __iter__(self):
        for priority_q_entry in self._heap:
            yield (priority_q_entry.item, priority_q_entry.priority)


'''
Created on Oct 17, 2015
@author: hari
'''
from __future__ import print_function
from builtins import range
from unittest import TestCase
from pycoils.trees.bst import BinarySearchTree
import unittest

class BST_Test_Empty_Tree(TestCase):    
    def setUp(self):
        self._bst = BinarySearchTree()
    
    def test_node_count_of_empty_tree(self):
        self.assertEqual(self._bst.node_count, 0, 'Empty tree node count must be 0')
        
    def test_find_key_empty_tree(self):
        self.assertEqual(self._bst.find(20), None, 'Empty tree find operation must return None')
    
    def test_delete_element_empty_tree(self):
        self._bst.remove(20)
    
    def tearDown(self):
        self._bst = None

class BST_Test_Tree_With_1_Element(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 10, obj = 10)
    
    def test_node_count_of_tree_with_1_node(self):
        self.assertEqual(self._bst.node_count, 1, 'Tree node count must be 1')
        
    def test_find_key_of_tree_with_1_node(self):
        self.assertEqual(self._bst.find(10), 10, 'Find operation failed on tree with 1 node')
    
    def test_delete_element_of_tree_with_1_node(self):
        self._bst.remove(10)
        self.assertEqual(self._bst.node_count, 0, 'Empty tree node count must be 0')
        self.assertEqual(self._bst.find(20), None, 'Empty tree find operation must return None')
            
    def tearDown(self):
        self._bst = None

class BST_Test_Tree_Count_Find_Element_With_10_Elements(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
        self._bst_node_count = 10
    
    def test_node_count_of_tree_with_10_node(self):
        self.assertEqual(self._bst.node_count, self._bst_node_count, 'Tree node count must be 10')
        
    def test_find_element_of_tree_with_10_node(self):
        for i in range(1, 11): # 1 to 10
            tree_element = self._bst.find(i)
            self.assertNotEqual(tree_element, None, 'BST findElement did not return existing element')
            self.assertEqual(tree_element, i, 'Find operation failed on tree with 1 node')
    
    def tearDown(self):
        self._bst = None
        
class BST_Test_Tree_Delete_Element_With_10_Elements(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
        self._bst_node_count = 10
        
    def test_delete_element_of_tree_non_existing_element(self):
         self._bst.remove(11)
         self.assertEqual(self._bst.node_count, self._bst_node_count, 'Tree node count must be 10')
    
    def test_delete_element_of_tree_with_10_node(self):
        elements_to_delete = [10, 1, 7, 3, 5, 8, 2, 6, 9]
        for element in elements_to_delete:
            self._bst.remove(element)
            self.assertEqual(self._bst.find(element), None, 'Element found in BST after deleting it!')
            self._bst_node_count = self._bst_node_count - 1
            self.assertEqual(self._bst.node_count, self._bst_node_count, 'Tree node count must tally after deletion')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Tree_Delete_Element_With_10_Sorted_Elements(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 6, obj = 6)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        
        self._bst_node_count = 10
        
    def test_delete_element_of_tree_non_existing_element(self):
        self._bst.remove(11)
        self.assertEqual(self._bst.node_count, self._bst_node_count, 'Tree node count must be 10')
    
    def test_delete_element_of_tree_with_10_node(self):
        elements_to_delete = [10, 1, 7, 3, 5, 8, 2, 6, 9, 4]
        for element in elements_to_delete: 
            self._bst.remove(element)
            self.assertEqual(self._bst.find(element), None, 'Element found in BST after deleting it!')
            self._bst_node_count = self._bst_node_count - 1
            self.assertEqual(self._bst.node_count, self._bst_node_count, 'Tree node count must tally after deletion')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Tree_Delete_Element_With_10_Reverse_Sorted_Elements(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 6, obj = 6)
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        
        self._bst_node_count = 10
        
    def test_delete_element_of_tree_non_existing_element(self):
        self._bst.remove(11)
        self.assertEqual(self._bst.node_count, self._bst_node_count, 'Tree node count must be 10')
    
    def test_delete_element_of_tree_with_10_node(self):
        elements_to_delete = [10, 1, 7, 3, 5, 8, 2, 6, 9, 4]
        for element in elements_to_delete: 
            self._bst.remove(element)
            self.assertEqual(self._bst.find(element), None, 'Element found in BST after deleting it!')
            self._bst_node_count = self._bst_node_count - 1
            self.assertEqual(self._bst.node_count, self._bst_node_count, 'Tree node count must tally after deletion')
    def tearDown(self):
        self._bst = None
        
class BST_Test_Has_Key_with_Empty_Tree(TestCase):    
    def setUp(self):
        self._bst = BinarySearchTree()
    
    def test_has_key_empty_tree(self):
        self.assertEqual(20 in self._bst, False, 'Empty tree has_key operation must return False')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Has_Key_with_Single_Node_Tree(TestCase):    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 10, obj = 'Object value')
    
    def test_has_key_empty_tree(self):
        self.assertEqual(10 in self._bst, True, 'Tree has_key operation must return True for existing key')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Has_Key_With_10_Elements(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
        self._bst_node_count = 10
    
    def test_has_key_with_10_node(self):
        for i in range(1, 11): # 1 to 10
            self.assertEqual(i in self._bst, True, 'has_key operation must return True for existing key.')
    
    def tearDown(self):
        self._bst = None

class BST_Test_Replace_Empty_Tree(TestCase):    
    def setUp(self):
        self._bst = BinarySearchTree()
    
    def test_replace_with_empty_tree(self):
        with self.assertRaises(KeyError):
            self._bst.replace('key', None)
        
    def tearDown(self):
        self._bst = None

class BST_Test_Replace_With_10_Elements_Non_Existing_Key(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
        self._bst_node_count = 10
    
    def test_replace_with_full_tree(self):
        with self.assertRaises(KeyError):
            self._bst.replace(-99, None)
        
    def tearDown(self):
        self._bst = None

class BST_Test_Replace_With_10_Elements_Existing_Key(TestCase):
    
    def setUp(self):
        self._bst = BinarySearchTree()
        self._bst.insert(key = 5, obj = 5)
        self._bst.insert(key = 8, obj = 8)
        self._bst.insert(key = 7, obj = 7)
        self._bst.insert(key = 9, obj = 9)
        self._bst.insert(key = 10, obj = 10)
        self._bst.insert(key = 2, obj = 2)
        self._bst.insert(key = 1, obj = 1)
        self._bst.insert(key = 3, obj = 3)
        self._bst.insert(key = 4, obj = 4)
        self._bst.insert(key = 6, obj = 6)
        
        self._bst_node_count = 10
    
    def test_replace_with_existing_key_on_tree(self):
        key_to_replace = 5
        value_to_set = -5
        
        self._bst.replace(key_to_replace, value_to_set)
        
        self.assertEqual(key_to_replace in self._bst, True, 'Tree has_key operation must return True for existing key')
        self.assertEqual(self._bst.find(key_to_replace), value_to_set, 'Find operation failed on tree after replace')
        self.assertNotEqual(self._bst.find(key_to_replace), key_to_replace, 'replaced key still exists!')
        
    def tearDown(self):
        self._bst = None

#Allows running as python run.
if __name__ == '__main__':
    print('BST tests')
    unittest.main()


'''
Created on Oct 25, 2015

@author: hari
'''
from __future__ import print_function
from builtins import str
from pycoils.trees.heap import Heap

if __name__ == '__main__':
    # A max heap. for a min heap use minHeap = True (is default)
    max_heap = Heap(min_heap = False)
    
    elements = [0, 4, 3, 2, 5, 1, 6]
    print('Adding elements to heap: %s' % elements)
    for element in elements:
        max_heap.add_item(element)
        
    print('Getting element from max heap')
    element = max_heap.get()
    print('Top element from heap is %s' % str(element))
    
    print('Iterating through heap') 
    for element in max_heap:
        print('heap popped %s' % str(element))
    
      


'''
Created on Nov 8, 2015

@author: hari
'''
from builtins import object
from functools import total_ordering

class WrappedString(object):
    '''
    This class simply wraps around python string.
    '''
    def __init__(self, string = 'default string'):
        self._string = string
    
    @property
    def wrapped_string(self):
        return self._string
    
    def __eq__(self, other):
        if other == None or type(other) != type(self):
            return False
        
        return self._string == other.wrapped_string
    
    def __lt__(self, other):
        if other == None or type(other) != type(self):
            return False
        
        return self._string < other.wrapped_string
    
    def __str__(self, *args, **kwargs):
        return self._string
    
    def __hash__(self, *args, **kwargs):
        return hash(self._string)


'''
Created on Oct 25, 2015
@author: hari
'''
from __future__ import print_function
from builtins import str
from pycoils.lists.singly_linked_list import LinkedList

def print_list(link_list):
    elements_in_list = []
    #iterate over the list
    for element in link_list:
        elements_in_list.append(element)
    print('List is now %s' % elements_in_list)
    #length of the list
    print('Length of the list is %s' % len(link_list))

def print_reverse_list(link_list):
    elements_in_list = []
    #iterate over the list
    for element in reversed(link_list):
        elements_in_list.append(element)
    print('Reversed List is now %s' % elements_in_list)
    #length of the list
    print('Length of the list is %s' % len(link_list))
    
    
if __name__ == '__main__':
    link_list = LinkedList()
    
    #add one element
    print('Adding element 1 to list')
    link_list.append(1)
    
    #we add the following list (iterable) of elements
    print('Adding elements 2 ,3 , 4, 5, 6, 7, 8, 9, 10')
    elements = [2 ,3 , 4, 5, 6, 7, 8, 9, 10]
    link_list.extend(elements)
    
    #head
    print('Head of list is: %s' % str(link_list.head))
    
    #tail
    print('Tail of list is: %s' % str(link_list.tail))
    print('Print list by iterating')
    
    #Find an element's index
    element = 4
    element_index = link_list.index(element)
    if element_index == -1:
        print('Element %s was not found on list' % str(element))
    else:
        print('Element %s found at index %s' % (str(element), str(element_index)))
        
    #Lets delete 7
    print('Deleting 7')
    element_to_remove = 7
    link_list.remove(element_to_remove)
    
    #Try to find it.
    print('Trying to find %s' % str(element_to_remove))
    index_of_deleted = link_list.index(element_to_remove)
    print('Index of deleted element is %s' % str(index_of_deleted))
    #print list again
    print_list(link_list)  
    
    #
    #Lets insert 7 at index 5
    print('Insert 7 into the list at index 4')
    element_to_insert = 7
    index_at = 5
    link_list.insert_at(index = index_at, element = element_to_insert)
    print_list(link_list)
    
    # Removing duplicates
    print('Removing duplicates')
    link_list = LinkedList()
    link_list.extend([1, 1, 2, 3, 4, 4, 5, 6, 6, 7, 8, 9, 10, 10])
    print_list(link_list)
    link_list.remove_duplicates()
    print_list(link_list)
    
    #Reverse print list using reversed
    print_reverse_list(link_list)


'''
Created on Oct 23, 2015
@author: hari
'''
#TESTS FOR SeperateChainHashTable with BSTHashBucket 
from builtins import range
import unittest
from pycoils.hashtables.HashTables import SeperateChainHashTable
from pycoils.hashtables.hashbuckets import BSTHashBucket

#want to use string keys and values. rather than hash (1, 17) 17 is initial capacity
list_of_strings_used_as_keys_and_values = ['one', 'two', 'three', 'four', 'five', 'six', 'seven', 'eight', 'nine', 'ten',
                                           'eleven', 'twelve', 'thirteen', 'fourteen', 'fifteen', 'sixteen', 'seventeen'] 
class SeperateChainHashTable_with_BSTHashBucket_Test_With_0_Elements(unittest.TestCase):
    
    def setUp(self):
        self._chained_hash_table = SeperateChainHashTable(bucket_type_class = BSTHashBucket)
    
    def test_get_empty_hash_table(self):
        non_existing_key = "SomeKey"
        self.assertEqual(self._chained_hash_table.get(key = non_existing_key), None, 'get on Non existing key must return None')
        with self.assertRaises(KeyError):
            self._chained_hash_table[non_existing_key]
    
    def test_del_empty_hash_table(self):
        non_existing_key = "SomeKey"
        try:
            del self._chained_hash_table[non_existing_key]
        except Exception as e:
            self.assertEqual(e.__class__.__name__, KeyError.__name__, 'deleting non-existing key did not raise key error')
    
    def test_len_empty_hash_table(self):
        self.assertEqual(len(self._chained_hash_table), 0, 'Empty hash table length must be 0')
    
    def test_has_key_empty_hash_table(self):
        non_existing_key = "SomeKey"
        self.assertEqual(self._chained_hash_table.has_key(key = non_existing_key), 0, 'Empty hash table length must be 0')
    
    def tearDown(self):
        self._chained_hash_table = None

class SeperateChainHashTable_with_BSTHashBucket_Test_With_1_Elements(unittest.TestCase):
    
    def setUp(self):
        self._chained_hash_table = SeperateChainHashTable(bucket_type_class = BSTHashBucket)
        self._key = 'some_key'
        self._value = 'some_value'
        self._chained_hash_table[self._key] = self._value
         
    def test_get_and_len_on_single_entry_hash_table(self):
        self.assertEqual(self._chained_hash_table.get(key = self._key), self._value, 'get on existing key failed')
        self.assertEqual(self._chained_hash_table[self._key], self._value, 'get on existing key failed')
    
        self.assertEqual(len(self._chained_hash_table), 1, 'single entry hash table length must be 1')
        
    def test_del_single_entry_hash_table(self):
        del self._chained_hash_table[self._key]
        try:
            del self._chained_hash_table[self._key]
        except Exception as e:
            self.assertEqual(e.__class__.__name__, KeyError.__name__, 'deleting non-existing key did not raise key error')
    
    def tearDown(self):
        self._chained_hash_table = None

class SeperateChainHashTable_with_BSTHashBucket_Test_Capacity_On_Resize(unittest.TestCase):
    def setUp(self):
        self._chained_hash_table = SeperateChainHashTable(bucket_type_class = BSTHashBucket)
        self._default_initial_capacity = 17
        for i in list_of_strings_used_as_keys_and_values:# 1 to 17 default initial capacity is 17
            self._chained_hash_table[i] = i
             
    def test_capacity_on_resize(self):
        for i in list_of_strings_used_as_keys_and_values:
            self.assertEqual(self._chained_hash_table.get(key = i), i, 'get on existing key failed')
            self.assertEqual(self._chained_hash_table[i], i, 'get on existing key failed')
        #would have resized but number of items would be same
        self.assertEqual(len(self._chained_hash_table), self._default_initial_capacity, 'hash table length does not add up')
        #but table size would have doubled
        self.assertEqual(self._chained_hash_table.current_capacity, self._default_initial_capacity * 2, 'hash table capacity does not add up')
                
    def tearDown(self):
        self._chained_hash_table = None
        
class SeperateChainHashTable_with_BSTHashBucket_Test_With_Full_Initial_Capacity_Elements_Forces_Resize(unittest.TestCase):
    def setUp(self):
        self._chained_hash_table = SeperateChainHashTable(bucket_type_class = BSTHashBucket)
        self._default_initial_capacity = 17
        for i in list_of_strings_used_as_keys_and_values:# 1 to 17 default initial capacity is 17
            self._chained_hash_table[i] = i
             
    def test_get_and_len_on_full_initial_capacity_hash_table(self):
        for i in list_of_strings_used_as_keys_and_values:
            self.assertEqual(self._chained_hash_table.get(key = i), i, 'get on existing key failed')
            self.assertEqual(self._chained_hash_table[i], i, 'get on existing key failed')
        #would have resized but number of items would be same
        self.assertEqual(len(self._chained_hash_table), self._default_initial_capacity, 'hash table length does not add up')
        #but table size would have doubled
        self.assertEqual(self._chained_hash_table.current_capacity, self._default_initial_capacity * 2, 'hash table capacity does not add up')
        
    def test_del_then_get_and_len_on_full_initial_capacity_hash_table(self):
        rolling_capacity = len(self._chained_hash_table)
        for i in list_of_strings_used_as_keys_and_values:
            del self._chained_hash_table[i]
            rolling_capacity = rolling_capacity - 1
            self.assertEqual(self._chained_hash_table.get(key = i), None, 'get on Non existing key must return None')
            with self.assertRaises(KeyError):
                self._chained_hash_table[i]
            self.assertEqual(len(self._chained_hash_table), rolling_capacity, 'hash table length did not add up after delete')
            
    def tearDown(self):
        self._chained_hash_table = None

class SeperateChainHashTable_with_BSTHashBucket_Test_With_Less_Than_Initial_Capacity_Elements_Cause_No_Resize(unittest.TestCase):
    '''
    By adding 5 elements to an initial capacity of 17 the load factor is just 0.2941 which is < 0.75 loadfactor threshold for resize.
    '''
    def setUp(self):
        self._chained_hash_table = SeperateChainHashTable(bucket_type_class = BSTHashBucket)
        self._default_initial_capacity = 5
        for i in list_of_strings_used_as_keys_and_values[0:5]:
            self._chained_hash_table[i] = i
             
    def test_get_and_len_on_less_than_initial_capacity_hash_table(self):
        for i in list_of_strings_used_as_keys_and_values[0:5]:
            self.assertEqual(self._chained_hash_table.get(key = i), i, 'get on existing key failed')
            self.assertEqual(self._chained_hash_table[i], i, 'get on existing key failed')
            
        self.assertEqual(len(self._chained_hash_table), self._default_initial_capacity, 'hash table length must be same as full initial capacity')
    
    def test_del_then_get_and_len_on_less_thaninitial_capacity_hash_table(self):
        rolling_capacity = self._default_initial_capacity
        for i in list_of_strings_used_as_keys_and_values[0:5]:
            del self._chained_hash_table[i]
            rolling_capacity = rolling_capacity - 1
            self.assertEqual(self._chained_hash_table.get(key = i), None, 'get on Non existing key must return None')
            with self.assertRaises(KeyError):
                self._chained_hash_table[i]
            self.assertEqual(len(self._chained_hash_table), rolling_capacity, 'hash table length did not add up after delete')
            
    def tearDown(self):
        self._chained_hash_table = None
        
class SeperateChainHashTable_with_BSTHashBucket_Test_Multiple_Resizes(unittest.TestCase):
    
    def setUp(self):
        self._chained_hash_table = SeperateChainHashTable(bucket_type_class = BSTHashBucket)
        self._default_initial_capacity = 17 
        self._length_upper_limit = 1001 #always choose some multiple of 100 + 1
        
    def test_get_and_len_with_a_truck_load_of_resizes(self):
        
        for i in range(1, self._length_upper_limit): # will trigger resize 
            self._chained_hash_table[i] = i

        for i in range(1, self._length_upper_limit): # will trigger 
            self.assertEqual(self._chained_hash_table.get(key = i), i, 'get on existing key failed')
            self.assertEqual(self._chained_hash_table[i], i, 'get on existing key failed')
        #would have resized but number of items would be same
        self.assertEqual(len(self._chained_hash_table), self._length_upper_limit - 1, 'hash table length does not add up')
        #but table size would have doubled
        current_capacity_would_be = 2176 #coz 13 is where a default size 17 table resizes at first.
        self.assertEqual(self._chained_hash_table.current_capacity, current_capacity_would_be,'hash table capacity does not add up')
    
    def test_del_then_get_and_len_with_a_truck_load_of_resizes(self):
        
        for i in range(1, self._length_upper_limit): # will trigger resize 
            self._chained_hash_table[i] = i
            
        rolling_length = len(self._chained_hash_table)
        for i in range(1, self._length_upper_limit): # will trigger
            del self._chained_hash_table[i]
            rolling_length = rolling_length - 1
            self.assertEqual(self._chained_hash_table.get(key = i), None, 'get on Non existing key must return None')
            with self.assertRaises(KeyError):
                self._chained_hash_table[i]
            self.assertEqual(len(self._chained_hash_table), rolling_length, 'hash table length did not add up after delete')
        
    def tearDown(self):
        self._chained_hash_table = None
#Allows running as python run.
if __name__ == '__main__':
    unittest.main()